runcorpusplots <- function() {
  library(ggplot2)
  source("util.R")
  source("plot.R")
  
  pathdir="D:/filter/data_pre"
  outdir="D:/filter/data_out/"
  assign("out.dir", outdir, envir = .GlobalEnv)
  files <- list.files(path=pathdir, pattern="*.csv", full.names=T, recursive=FALSE)
  lapply(files,processcsv)
}

compareont<-function(o1,o2,outdir){
  csv<-paste(getwd(),out.dir.plot,"compare_out.csv",sep="")
  unlink(csv)
  command<-paste("compare_ontologies.jar",o1,o2,outdir,csv)
  system(paste("java","-jar",command))
  d<-csvdata(csv)
  return(d)
}

csvdata<-function(f){
  read.csv(file=f,head=TRUE,sep=",")
}

processcsv <- function(x) {
  print(x)
  csvfilename <- basename(x)
  outdirfile <- file.path(out.dir.plot, sub("[.]csv","",csvfilename))
  dir.create(outdirfile, showWarnings = FALSE)
  assign("out.dir.plot", paste(outdirfile,"/",sep=""), envir = .GlobalEnv)
  generateCorpusPlots(read.csv(file=x,head=TRUE,sep=","))
}

generateCorpusDescriptionPlot<-function(metrics){
  all_metr_for_loghist <- c("ax_o_abox_size_incl","ax_o_tbox_size_incl")
  metrics[, all_metr_for_loghist] <- sapply(metrics[, all_metr_for_loghist], as.numeric)
  metricsfiltered <- subset(metrics,ax_o_tbox_size_incl>=0)
  
  ptbox<-ggplot(metricsfiltered, aes(ax_o_tbox_size_incl))+ ggtitle("TBox size") + scale_x_log10(breaks=c(10,100,1000,10000,100000,1000000,10000000),labels=c(10,100,1000,10000,100000,1000000,10000000)) + geom_histogram(binwidth=log10(2))+ theme(plot.margin=unit(c(0,0,0,0), "cm"),plot.title = element_text(vjust=-3,hjust=0.8,lineheight=.8, face="bold",size=17),legend.position="none",axis.title.y = element_blank(),axis.title.x = element_blank(),axis.text.x  = element_text(size=13),axis.text.y  = element_text(size=15))
  pabox<-ggplot(metricsfiltered, aes(ax_o_abox_size_incl))+ ggtitle("ABox size") + scale_x_log10(breaks=c(10,100,1000,10000,100000,1000000,10000000),labels=c(10,100,1000,10000,100000,1000000,10000000)) + geom_histogram(binwidth=log10(2))+ theme(plot.margin=unit(c(0,0,0,0), "cm"),plot.title = element_text(vjust=-3,hjust=0.8,lineheight=.8, face="bold",size=17),legend.position="none",axis.title.y = element_blank(),axis.title.x = element_blank(),axis.text.x  = element_text(size=13),axis.text.y  = element_text(size=15))
  
  pprofile<-generateDLProfileBarchart(metricsfiltered)$plot_1+ ggtitle("OWL 2 Profiles")+ theme(plot.title = element_text(vjust=-3,hjust=0.6,lineheight=.8, face="bold",size=17),plot.margin=unit(c(0,0,0,0), "cm"),legend.position="none",axis.title.y = element_blank(),axis.title.x = element_blank(),axis.text.x  = element_text(size=15),axis.text.y  = element_text(size=15))
  #paxtype<-generateAxiomtypeProfileBarchart(metricsfiltered)$plot_1+ theme(plot.margin=unit(c(0.7,0,0,0), "cm"),plot.title = element_text(vjust=-5,hjust=2,lineheight=.8, face="bold",size=17),legend.position="none",axis.title.y = element_blank(),axis.title.x = element_blank(),axis.text.x  = element_text(size=13),axis.text.y  = element_text(size=15))
  pconstructs<-generateConstructsBarchart(metricsfiltered)$plot_1+ ggtitle("Logical constructs")+ theme(plot.margin=unit(c(0,0,0,0), "cm"),plot.title = element_text(vjust=-2,lineheight=.8, face="bold",size=17),legend.position="none",axis.title.y = element_blank(),axis.title.x = element_blank(),axis.text.x  = element_text(size=15),axis.text.y  = element_text(size=15))
  
  #pleft<-arrangeGrob(pprofile,pconstructs,ncol=1)
  #pdown<-arrangeGrob(ptbox,pabox,ncol=2)
  #pup<-arrangeGrob(paxtype,pleft,ncol=2)
  p<-arrangeGrob(ptbox,pabox,pprofile,pconstructs,ncol=2, heights=c(2/4, 2/4))
  return(p)
}

generateViolationPlots <- function(datavector) {
  hbindeclarationfailure<-c("viol_useofundeclaredannotationproperty","viol_useofundeclaredclass","viol_useofundeclareddataproperty","viol_useofundeclareddatatype","viol_useofundeclaredobjectproperty")
  hbinmisusedreserved<-c("viol_useofreservedvocabularyforannotationpropertyiri","viol_useofreservedvocabularyforclassiri","viol_useofreservedvocabularyfordatapropertyiri","viol_useofreservedvocabularyforindividualiri","viol_useofreservedvocabularyforobjectpropertyiri","viol_useofreservedvocabularyforontologyiri")
  hbinsyntactic<-c("viol_insufficientindividuals","viol_insufficientoperands","viol_insufficientpropertyexpressions","viol_useofnonabsoluteiri","viol_ontologyirinotabsolute")
  hbindlviolation<-c("viol_cycleindatatypedefinition","viol_useofunknowndatatype","viol_useoftopdatapropertyassubpropertyinsubpropertyaxiom","viol_useofnonsimplepropertyindisjointpropertiesaxiom","viol_useofnonsimplepropertyinfunctionalpropertyaxiom","viol_useofnonsimplepropertyininversefunctionalobjectpropertyaxiom","viol_useofnonsimplepropertyinirreflexivepropertyaxiom","viol_useofpropertyinchaincausescycle","viol_useofnonsimplepropertyincardinalityrestriction","viol_useofnonsimplepropertyinasymmetricobjectpropertyaxiom","viol_datatypeirialsousedasclassiri","viol_illegalpunning","viol_lexicalnotinlexicalspace")
  
  exportplots(plotViolationBin(datavector,hbindeclarationfailure,"profile_Violations_declarationfailure"))
  exportplots(plotViolationBin(datavector,hbinmisusedreserved,"profile_Violations_misused_vocabulary"))
  exportplots(plotViolationBin(datavector,hbinsyntactic,"profile_Violations_syntactic"))
  exportplots(plotViolationBin(datavector,hbindlviolation,"profile_Violations_inherentdl"))
  return("doneviolationplots")
}

plotViolationBin <- function (datavector,columns,binname) {
  hbin <- columns[columns %in% names(datavector)] 
  if(length(hbin)<1) {
    return(emptyplot())
  }
  bin <- datavector[hbin]
  bin[is.na(bin)] <- 0
  count <- colSums(bin > 0)
  labels <- hbin
  return(plotbarchart_horiz(mergecolumnstodataframe(count,labels,c("ontology_ct","violation")),"violation","ontology_ct",binname))
}

generateProfileMetrics <- function(m){
  m$profiled<-ifelse(m$profile_owl2_el==1|m$profile_owl2_ql==1|m$profile_owl2_rl==1,TRUE,FALSE)
  m$owlfull<-ifelse(m$profile_owl2_dl==0,TRUE,FALSE)
  m$dl_pure<-ifelse(m$profile_owl2_el==0&m$profile_owl2_ql==0&m$profile_owl2_rl==0&m$profile_owl2_dl==1,TRUE,FALSE)
        
  m$rdfs<-ifelse(m$profile_rdfs==1,TRUE,FALSE)
  m$domrangsum<-rowSums(m[, c("axtct_incl_objectpropertyrange","axtct_incl_objectpropertydomain","axtct_incl_datapropertyrange","axtct_incl_datapropertydomain")],na.rm=TRUE)
  m$rdfs_hier<-ifelse(m$rdfs&m$domrangsum==0,TRUE,FALSE)
            
  m$el_pure<-ifelse(m$profiled==1&m$profile_owl2_el==1&m$profile_owl2_ql==0&m$profile_owl2_rl==0,TRUE,FALSE)
  m$ql_pure<-ifelse(m$profiled==1&m$profile_owl2_el==0&m$profile_owl2_ql==1&m$profile_owl2_rl==0,TRUE,FALSE)
  m$rl_pure<-ifelse(m$profiled==1&m$profile_owl2_el==0&m$profile_owl2_ql==0&m$profile_owl2_rl==1,TRUE,FALSE)
  m$rl_el_ql<-ifelse(m$profiled==1&m$profile_owl2_el==1&m$profile_owl2_ql==1&m$profile_owl2_rl==1,TRUE,FALSE)
  m$rl_el<-ifelse(m$profiled==1&m$profile_owl2_el==1&m$profile_owl2_ql==0&m$profile_owl2_rl==1,TRUE,FALSE)
  m$rl_ql<-ifelse(m$profiled==1&m$profile_owl2_el==0&m$profile_owl2_ql==1&m$profile_owl2_rl==1,TRUE,FALSE)
  m$el_ql<-ifelse(m$profiled==1&m$profile_owl2_el==1&m$profile_owl2_ql==1&m$profile_owl2_rl==0,TRUE,FALSE)
  
  m$cat_main<-ifelse(m$owlfull,"OWL Full",ifelse(m$dl_pure,"Pure DL",ifelse(m$profiled,"Profiled",NA)))
  m$cat_prof<-ifelse(m$el_pure,"EL",ifelse(m$ql_pure,"QL",ifelse(m$rl_pure,"RL",
                ifelse(m$el_ql,"EL+QL",ifelse(m$rl_el,"EL+RL",ifelse(m$rl_ql,"RL+QL",
                ifelse(m$rl_el_ql,"EL+QL+RL",NA)))))))
  m$cat_rdfs<-ifelse(m$rdfs&m$rdfs_hier,"RDFS(-)",ifelse(m$rdfs,"RDFS","NOT RDFS"))
  
  return(m)
}

generateDLProfileBarchart <- function(m){
  print("Warning, fixing wrong RDFS (DL Pure + RDFS = DL Pure + Not RDFS)")
  m$cat_rdfs2<-ifelse(m$cat_main=="Pure DL"&m$cat_rdfs=="RDFS","NOT RDFS",m$cat_rdfs)
  p_main<-ggplot(m,aes(cat_main,fill=cat_rdfs2)) + geom_bar() + scale_fill_grey(start = .3, end = .7) + xlab("")+ylab("") + theme(axis.text=element_text(size=14), legend.position="none")
  p_prof<-ggplot(m[!is.na(m$cat_prof), ],aes(cat_prof,fill=cat_rdfs2)) + geom_bar()+ scale_fill_grey(start = .3, end = .7) + xlab("") +ylab("")+theme(axis.text=element_text(size=14),legend.justification=c(1,1), legend.position=c(1,1))
  p.plot<-arrangeGrob(p_main,p_prof,ncol=2,widths=c(1/3,2/3))
  return(list("plot_1"=p.plot, "function_name"="dl_profiles"))
}

generateAxiomtypeProfileBarchart <- function(metricsfiltered){
  count <- c(sum(metricsfiltered$axt_incl_AnnotationAssertion==1, na.rm=T),sum(metricsfiltered$axt_incl_AnnotationPropertyDomain==1, na.rm=T),sum(metricsfiltered$axt_incl_ClassAssertion==1, na.rm=T),sum(metricsfiltered$axt_incl_DataPropertyAssertion==1, na.rm=T),sum(metricsfiltered$axt_incl_DataPropertyDomain==1, na.rm=T),sum(metricsfiltered$axt_incl_DataPropertyRange==1, na.rm=T),sum(metricsfiltered$axt_incl_Declaration==1, na.rm=T),sum(metricsfiltered$axt_incl_DifferentIndividuals==1, na.rm=T),sum(metricsfiltered$axt_incl_DisjointClasses==1, na.rm=T),sum(metricsfiltered$axt_incl_EquivalentClasses==1, na.rm=T),sum(metricsfiltered$axt_incl_EquivalentDataProperties==1, na.rm=T),sum(metricsfiltered$axt_incl_EquivalentObjectProperties==1, na.rm=T),sum(metricsfiltered$axt_incl_FunctionalDataProperty==1, na.rm=T),sum(metricsfiltered$axt_incl_FunctionalObjectProperty==1, na.rm=T),sum(metricsfiltered$axt_incl_InverseFunctionalObjectProperty==1, na.rm=T),sum(metricsfiltered$axt_incl_InverseObjectProperties==1, na.rm=T),sum(metricsfiltered$axt_incl_ObjectPropertyAssertion==1, na.rm=T),sum(metricsfiltered$axt_incl_ObjectPropertyDomain==1, na.rm=T),sum(metricsfiltered$axt_incl_ObjectPropertyRange==1, na.rm=T),sum(metricsfiltered$axt_incl_SameIndividual==1, na.rm=T),sum(metricsfiltered$axt_incl_SubAnnotationPropertyOf==1, na.rm=T),sum(metricsfiltered$axt_incl_SubClassOf==1, na.rm=T),sum(metricsfiltered$axt_incl_SubDataPropertyOf==1, na.rm=T),sum(metricsfiltered$axt_incl_SubObjectPropertyOf==1, na.rm=T),sum(metricsfiltered$axt_incl_SymmetricObjectProperty==1, na.rm=T),sum(metricsfiltered$axt_incl_TransitiveObjectProperty==1, na.rm=T))
  labels <- c("AnnotationAssertion","AnnotationPropertyDomain","ClassAssertion","DataPropertyAssertion","DataPropertyDomain","DataPropertyRange","Declaration","DifferentIndividuals","DisjointClasses","EquivalentClasses","EquivalentDataProperties","EquivalentObjectProperties","FunctionalDataProperty","FunctionalObjectProperty","InverseFunctionalObjectProperty","InverseObjectProperties","ObjectPropertyAssertion","ObjectPropertyDomain","ObjectPropertyRange","SameIndividual","SubAnnotationPropertyOf","SubClassOf","SubDataPropertyOf","SubObjectPropertyOf","SymmetricObjectProperty","TransitiveObjectProperty")
  return(plotbarchart_horiz(mergecolumnstodataframe(count,labels,c("ontology_ct","axiomtype")),"axiomtype","ontology_ct","owl2_axiomtypes"))
}

generateConstructsBarchart <- function(metricsfiltered){
  count <- c(sum(metricsfiltered$constr_incl_AL==1, na.rm=T),sum(metricsfiltered$constr_incl_C==1, na.rm=T),sum(metricsfiltered$constr_incl_D==1, na.rm=T),sum(metricsfiltered$constr_incl_E==1, na.rm=T),sum(metricsfiltered$constr_incl_H==1, na.rm=T),sum(metricsfiltered$constr_incl_I==1, na.rm=T),sum(metricsfiltered$constr_incl_N==1, na.rm=T),sum(metricsfiltered$constr_incl_O==1, na.rm=T),sum(metricsfiltered$constr_incl_Q==1, na.rm=T),sum(metricsfiltered$constr_incl_S==1, na.rm=T),sum(metricsfiltered$constr_incl_TRAN==1, na.rm=T),sum(metricsfiltered$constr_incl_U==1, na.rm=T))
  labels <- c("AL","C","D","E","H","I","N","O","Q","S","TRAN","U")
  return(plotbarchart(mergecolumnstodataframe(count,labels,c("ontology_ct","construct")),"construct","ontology_ct","owl2_constructs"))
}

generateSummaryTable <- function(metricsfiltered,export=FALSE){
  plotfile <- paste(out.dir.plot,"summarystats",".csv",sep="")
  library(psych)
  subs <- c("filesize","ent_obj_property_count_incl","ent_obj_property_count","ent_indiv_count_incl","ent_indiv_count","ent_datatypes_count_incl","ent_datatypes_count","ent_dataproperty_count_incl","ent_dataproperty_count","ent_class_count_incl","ent_class_count","axtct_transitiveobjectproperty","axtct_symmetricobjectproperty","axtct_subpropertychainof","axtct_subobjectpropertyof","axtct_subdatapropertyof","axtct_subclassof","axtct_subannotationpropertyof","axtct_sameindividual","axtct_rule","axtct_reflexiveobjectproperty","axtct_objectpropertyrange","axtct_objectpropertydomain","axtct_objectpropertyassertion","axtct_irrefexiveobjectproperty","axtct_inverseobjectproperties","axtct_inversefunctionalobjectproperty","axtct_incl_transitiveobjectproperty","axtct_incl_symmetricobjectproperty","axtct_incl_subpropertychainof","axtct_incl_subobjectpropertyof","axtct_incl_subdatapropertyof","axtct_incl_subclassof","axtct_incl_subannotationpropertyof","axtct_incl_sameindividual","axtct_incl_rule","axtct_incl_reflexiveobjectproperty","axtct_incl_objectpropertyrange","axtct_incl_objectpropertydomain","axtct_incl_objectpropertyassertion","axtct_incl_irrefexiveobjectproperty","axtct_incl_inverseobjectproperties","axtct_incl_inversefunctionalobjectproperty","axtct_incl_functionalobjectproperty","axtct_incl_functionaldataproperty","axtct_incl_equivalentobjectproperties","axtct_incl_equivalentdataproperties","axtct_incl_equivalentclasses","axtct_incl_disjointunion","axtct_incl_disjointobjectproperties","axtct_incl_disjointclasses","axtct_incl_differentindividuals","axtct_incl_declaration","axtct_incl_datatypedefinition","axtct_incl_datapropertyrange","axtct_incl_datapropertydomain","axtct_incl_datapropertyassertion","axtct_incl_classassertion","axtct_incl_asymmetricobjectproperty","axtct_incl_annotationpropertyrangeof","axtct_incl_annotationpropertydomain","axtct_incl_annotationassertion","axtct_functionalobjectproperty","axtct_functionaldataproperty","axtct_equivalentobjectproperties","axtct_equivalentdataproperties","axtct_equivalentclasses","axtct_disjointunion","axtct_disjointobjectproperties","axtct_disjointclasses","axtct_differentindividuals","axtct_declaration","axtct_datatypedefinition","axtct_datapropertyrange","axtct_datapropertydomain","axtct_datapropertyassertion","axtct_classassertion","axtct_asymmetricobjectproperty","axtct_annotationpropertyrangeof","axtct_annotationpropertydomain","axtct_annotationassertion","ax_o_tbox_size_incl","ax_o_tbox_size","ax_o_rule_ct_incl","ax_o_rule_ct","ax_o_logical_axiom_count_incl","ax_o_logical_axiom_count","ax_o_abox_size_incl","ax_o_abox_size")
  data <- describe(metricsfiltered)
  rotate <- function(x) t(apply(x, 2, rev))
  rotateddata <- rotate(data)
  out<-t(as.matrix(rotateddata))
  if(export) {
  write.csv(out, file=plotfile) 
  }
  return(out)
}

generateCorpusPlots <- function(metrics){
  all_metr_for_loghist <- c("ax_o_logical_axiom_count","ax_o_logical_axiom_count_incl","ax_o_abox_size","ax_o_abox_size_incl","ax_o_tbox_size","ax_o_tbox_size_incl")
  metrics[, all_metr_for_loghist] <- sapply(metrics[, all_metr_for_loghist], as.numeric)
  metricsfiltered <- subset(metrics,ax_o_tbox_size_incl>=0)
  for(m in all_metr_for_loghist) {
    exportplots(plotlogaxhistogram(metricsfiltered,m))
  }
  #generateHistogramLogaxOverlap(metricsfiltered$ax_o_logical_axiom_count,metricsfiltered$ax_o_logical_axiom_count_incl,outdir,"Logical Axioms","Nr. of Axioms")
  #generateHistogramLogaxOverlap(metricsfiltered$ax_o_abox_size,metricsfiltered$ax_o_abox_size_incl,outdir,"ABox Axioms","Nr. of Axioms")
  #generateHistogramLogaxOverlap(metricsfiltered$ax_o_tbox_size,metricsfiltered$ax_o_tbox_size_incl,outdir,"TBox Axioms","Nr. of Axioms")
  exportplots(generateDLProfileBarchart(metricsfiltered))
  generateViolationPlots(metricsfiltered)
  exportplots(generateAxiomtypeProfileBarchart(metricsfiltered))
  exportplots(generateConstructsBarchart(metricsfiltered))
  generateSummaryTable(metricsfiltered)
}

appendconsistency<-function(corpus_md,consistencyf) {
  consistency<-read.csv(file=consistencyf,head=TRUE,sep=",")
  consistency$filename<-gsub("obi_bcgo", "obi-bcgo", consistency$filename)
  consistency$o<-get_ontology_short_name(consistency$filename)
  consis<-consistency[c("consistent","o","consistencytime","param_reasoner")]
  names(consis)<-c("c","o","ct","r")
  consis_w<-(reshape(consis, timevar = "r", idvar = c("o"), direction = "wide"))
  consis_w$ctnna <- apply(consis_w[,c("c.hermit","c.jfact","c.pellet")], 1, function(x) { length(x[!is.na(x)]) } )
  consis_w$sum <- apply(consis_w[,c("c.hermit","c.jfact","c.pellet")], 1, function(x) { sum(x[!is.na(x)]) } )
  consis_w$o_consistent<-ifelse(consis_w$ctnna==0,"unkown",ifelse(consis_w$ctnna==1,ifelse(consis_w$sum==1,"weakcon","weakin"),ifelse(consis_w$sum==0,"inconsistent",ifelse(consis_w$sum>=2,"consistent",NA))))
  
  corpus_md<-merge(corpus_md,consis_w[c("o","o_consistent")],by="o",all.x=TRUE)
  return(corpus_md)
}