dropboxdir<-"C:/Dropbox"
wd<-paste(dropboxdir,"/PHD/datanalysis/experimentpipelines/",sep="")
setwd(wd)
source("util.R")
prepareworkspace("phd_bijanconjecture")

reasoner<-c("fact","hermit","pellet")
datadir<-paste(dropboxdir,"/PHD/Data/experiment_results/bijanconjecture/00finaldata/",sep="")

ontmetadatafile<-paste(datadir,"bpjan2015_phd_finalmetrics.csv",sep="")
baselinef<-paste(datadir,"phd_baseline.csv",sep="")
consistencyf<-paste(datadir,"phd_consistency.csv",sep="")

subset_run1f<-paste(datadir,"phd_bijanconjecture_subsets_run1.csv",sep="")
subset_run2f<-paste(datadir,"phd_bijanconjecture_subsets_run2.csv",sep="")
subset_run3f<-paste(datadir,"phd_bijanconjecture_subsets_run3.csv",sep="")
subset_pathf<-paste(datadir,"phd_bijanconjecture_subsets_path.csv",sep="")
subset_path_analysisf<-paste(datadir,"phd_bijanconjecture_paths.csv",sep="")
subset_path_regionsf<-paste(datadir,"phd_bijanconjecture_hardregions.csv",sep="")
subset_modulesf<-paste(datadir,"phd_bijanconjecture_modules.csv",sep="")
realmodulesf<-paste(datadir,"phd_bijanconjecture_realmods.csv",sep="")

# PREPARE DATA #
## Ontology Metadata ##
corpus_md<-create_corpus_dataset(ontmetadatafile)
corpus_md$o<-gsub("obi_bcgo", "obi-bcgo", corpus_md$o)
corpus_md$filename<-gsub("obi_bcgo", "obi-bcgo", corpus_md$filename.x)
corpus_md<-appendconsistency(corpus_md,consistencyf)


## PATHS ##
subset_path<-read.csv(file=subset_pathf,head=TRUE,sep=",")
subset_path$filename<-gsub("obi_bcgo", "obi-bcgo", subset_path$filename)
subset_path$o<-getontnamefromlongname(subset_path$filename,7,TRUE)
subset_path$path<-getontnamefromlongname(subset_path$filename,2,TRUE)
subset_path$sub_i<-getontnamefromlongname(subset_path$filename,4,TRUE)
subset_path$sub_tot<-getontnamefromlongname(subset_path$filename,5,TRUE)
subset_path<-subset_path[subset_path$o %in% unique(corpus_md$o),]
subset_path$r<-subset_path$param_reasoner
subset_path$region<-paste(subset_path$o,subset_path$r,subset_path$sub_i,sep="_")

### Exclude jfact
subset_path<-subset_path[subset_path$param_reasoner %in% reasoner,]

subset_path_fail<-subset_path[is.na(subset_path$consistent),]
subset_path<-subset_path[!is.na(subset_path$consistent),]

### Baseline ### (8 in paths)
baseline<-read.csv(file=baselinef,head=TRUE,sep=",")
baseline$filename<-gsub("obi_bcgo", "obi-bcgo", baseline$filename)
baseline$o<-get_ontology_short_name(baseline$filename)
baseline<-baseline[baseline$o %in% unique(subset_path$o),]
baseline<-baseline[baseline$param_reasoner %in% reasoner,]
baseline_fail<-baseline[is.na(baseline$consistent),]
baseline<-baseline[!is.na(baseline$consistent),]
baseline$sub_i<-8
baseline$path<-0
baseline$r<-baseline$param_reasoner

### Regions ###
subset_path_regions<-read.csv(file=subset_path_regionsf,head=TRUE,sep=",")
subset_path_regions$ontology<-gsub("obi_bcgo", "obi-bcgo", subset_path_regions$ontology)
subset_path_regions$o<-get_ontology_short_name(subset_path_regions$ontology)
subset_path_regions<-subset_path_regions[subset_path_regions$reasoner %in% reasoner,]

### Merge in cltime O ###
baseline$cltime_o<-baseline$classificationtime
baseline$size_o<-baseline$logical_axiom_count_incl
subset_path<-merge(subset_path,baseline[c("o","r","cltime_o")],by=c("o","r"),all.x=TRUE)
subset_path<-merge(subset_path,corpus_md[c("o","cat_main")],by=c("o"),all.x=TRUE)
subset_path$tooeasy<-ifelse(subset_path$classificationtime<10000000000,TRUE,FALSE)
subset_path$hardregion<-ifelse(subset_path$cltime_o<subset_path$classificationtime,TRUE,FALSE)

count_region_wit<-count(subset_path[!is.na(subset_path$cltime_o)&subset_path$hardregion&subset_path$cat_main!="OWL Full",][c("sub_i")])
count_region_wit$rel<-round(count_region_wit$freq/sum(count_region_wit$freq)*100,2)
count_region_wit_e<-count(subset_path[subset_path$tooeasy&!is.na(subset_path$cltime_o)&subset_path$hardregion&subset_path$cat_main!="OWL Full",][c("tooeasy","sub_i")])
count_region_wit_e$rel<-round(count_region_wit_e$freq/sum(count_region_wit_e$freq)*100,2)
count_region_wit_h<-count(subset_path[!subset_path$tooeasy&!is.na(subset_path$cltime_o)&subset_path$hardregion&subset_path$cat_main!="OWL Full",][c("tooeasy","sub_i")])
count_region_wit_h$rel<-round(count_region_wit_h$freq/sum(count_region_wit_h$freq)*100,2)
names(count_region_wit)<-c("i","f_all","r_all")
names(count_region_wit_e)<-c("te","i","f_e","r_e")
names(count_region_wit_h)<-c("te","i","f_h","r_h")
count_region_merge<-merge(count_region_wit[c("i","f_all","r_all")],count_region_wit_h[c("i","f_h","r_h")],by="i",all=TRUE)
count_region_merge<-merge(count_region_merge,count_region_wit_e[c("i","f_e","r_e")],by="i",all=TRUE)
count_region_merge<-count_region_merge[c("i","f_all","f_h","f_e","r_all","r_h","r_e")]

#print(xtable(count_region_merge,digits=c(0,0,0,0,0,0,0,0)),include.rownames=FALSE)

### Prepare Region data frames ###
x<-subset_path[c("o","r","sub_i","classificationtime","hardregion","tooeasy")]
x<-x[complete.cases(x),]
#regions<-x
hard_regions<-unique(x[!x$tooeasy&x$hardregion,][c("o","r","sub_i")])
hard_regions$id<-paste(hard_regions$o,hard_regions$sub_i,sep="_")
nothard_regions<-unique(x[!x$tooeasy&!x$hardregion,][c("o","r","sub_i")])
nothard_regions$id<-paste(nothard_regions$o,nothard_regions$sub_i,sep="_")

hard_regions_e<-unique(x[x$tooeasy&x$hardregion,][c("o","r","sub_i")])
hard_regions_e$id<-paste(hard_regions_e$o,hard_regions_e$sub_i,sep="_")
nothard_regions_e<-unique(x[x$tooeasy&!x$hardregion,][c("o","r","sub_i")])
nothard_regions_e$id<-paste(nothard_regions_e$o,nothard_regions_e$sub_i,sep="_")

hard_regions_all<-unique(x[x$hardregion,][c("o","r","sub_i")])
hard_regions_all$id<-paste(hard_regions_all$o,hard_regions_all$sub_i,sep="_")
nothard_regions_all<-unique(x[!x$hardregion,][c("o","r","sub_i")])
nothard_regions_all$id<-paste(nothard_regions_all$o,nothard_regions_all$sub_i,sep="_")

count_region_witnesses<-count(x[c("hardregion","tooeasy")])
count_region_witnesses$rel<-round((count_region_witnesses$freq/nrow(x))*100,2)

## HARD SUBSET 20 ##
subset_run1<-read.csv(file=subset_run1f,head=TRUE,sep=",")
subset_run1$run<-"run1"
subset_run2<-read.csv(file=subset_run2f,head=TRUE,sep=",")
subset_run2$run<-"run2"
subset_run3<-read.csv(file=subset_run3f,head=TRUE,sep=",")
subset_run3$run<-"run3"
subset_run3$exception_message<-NA
subset_run3$exception<-NA

### DF of hard modules: names_hardsubsets[c("r","filename")] ###
x<-subset_run3
x$r<-x$param_reasoner
x$o<-getontnamefromlongname(x$filename,5,TRUE)
x<-merge(x,baseline[c("o","r","cltime_o")],by=c("o","r"),all.x=TRUE)
x$hardsubset<-ifelse(x$cltime_o<x$classificationtime,TRUE,FALSE)
names_hardsubsets<-unique(x[x$hardsubset,][c("r","filename")])
names_hardsubsets$hardsu<-TRUE

### Create DF for HS 20 subset: subset_o, subset_all, subset_all_hard ###
subset_all<-rbind(subset_run1,subset_run2,subset_run3)

subset_all$filename<-gsub("obi_bcgo", "obi-bcgo", subset_all$filename)

### Distinguish classification times for O and for subsets ####
subset_o<-subset_all[!grepl("^randomsubset",subset_all$filename),]

subset_all<-subset_all[grepl("^randomsubset",subset_all$filename),]
subset_all$sub_i<-getontnamefromlongname(subset_all$filename,2,TRUE)
subset_all$sub_tot<-getontnamefromlongname(subset_all$filename,3,TRUE)
subset_all$r<-subset_all$param_reasoner
subset_all$o<-getontnamefromlongname(subset_all$filename,5,TRUE)

subset_all<-merge(subset_all,baseline[c("o","r","cltime_o","size_o")],by=c("o","r"),all.x=TRUE)
subset_all<-merge(subset_all,corpus_md[c("o","cat_main","o_consistent","cat_prof","sizebin")],by="o",all.x=TRUE)

subset_all$hardsubset<-ifelse(subset_all$cltime_o<subset_all$classificationtime,TRUE,FALSE)
subset_all$tooeasy<-ifelse(subset_all$classificationtime<10000000000,TRUE,FALSE)
subset_all$hardregion<-ifelse(subset_all$cltime_o<subset_all$classificationtime,TRUE,FALSE)
subset_all$region<-paste(subset_all$o,subset_all$r,subset_all$sub_i,sep="_")

### Remove all JFact results ### (every hard subset that was evidenced in a JFact hard region) ---- Actually there werent even any!
subset_all$regionid<-paste(subset_all$o,subset_all$sub_i,sep="_")
subset_all_jfact<-subset_all[!(subset_all$regionid %in% unique(hard_regions$id)),]
subset_all<-subset_all[subset_all$regionid %in% unique(hard_regions$id),]

### exclude illegal ontologies ###
subset_all<-subset_all[subset_all$o %in% unique(corpus_md$o),]

### exclude jfact ###
subset_all<-subset_all[subset_all$param_reasoner %in% reasoner,]

subset_all_fail<-subset_all[is.na(subset_all$consistent),]
subset_all<-subset_all[!is.na(subset_all$consistent),]

### Full ontology measurements DF preprartion ###
subset_o$o<-get_ontology_short_name(subset_o$filename)
subset_o$r<-subset_o$param_reasoner
#### Exlude illegal from o ####
subset_o<-subset_o[subset_o$o %in% unique(corpus_md$o),]
subset_o<-subset_o[subset_o$param_reasoner %in% reasoner,]

subset_o_fail<-subset_o[is.na(subset_o$consistent),]
subset_o<-subset_o[!is.na(subset_o$consistent),]
subset_o$cltime_o<-subset_o$classificationtime

subset_all_hard<-merge(subset_all, names_hardsubsets, by=c("r","filename"), all.x=TRUE)
subset_all_hard<-subset_all_hard[!is.na(subset_all_hard$hardsu),]
subset_all_hard_unique<-unique(subset_all_hard[c("r","o","sub_i","filename","size_o","sizebin","logical_axiom_count_incl","tbox_size_incl","abox_size_incl","expressivity_incl","cat_main")])
subset_all_hard_unique$om_diff_size<-subset_all_hard_unique$size_o-subset_all_hard_unique$logical_axiom_count_incl
subset_all_hard_unique$om_diff_size_rel<-round(100*(subset_all_hard_unique$om_diff_size/subset_all_hard_unique$size_o),1)
#subset_all_hard_unique$exp_cat<-ifelse(subset_all_hard_unique$owl2_dl==0,"OWL Full",ifelse(subset_all_hard_unique$owl2_el==1|subset_all_hard_unique$owl2_rl==1|subset_all_hard_unique$owl2_ql==1,"Profiled","Pure DL"))
subset_all_hard_unique$uhard<-TRUE

subset_o<-merge(subset_o,unique(subset_all_hard_unique[c("o","r","uhard")]),by=c("o","r"),all.x=TRUE)
subset_o<-subset_o[subset_o$uhard&!is.na(subset_o$uhard),]

### Create Path Vector: sub_path_wide ###
sub_path_df<-rbind(baseline[c("param_reasoner","classificationtime","o","sub_i","path")],subset_path[c("param_reasoner","classificationtime","o","sub_i","path")])
sub_path_df<-rbind(baseline[c("param_reasoner","classificationtime","o","sub_i","path")],subset_path[c("param_reasoner","classificationtime","o","sub_i","path")])
baseline$path<-1
sub_path_df<-rbind(baseline[c("param_reasoner","classificationtime","o","sub_i","path")],sub_path_df)
baseline$path<-2
sub_path_df<-rbind(baseline[c("param_reasoner","classificationtime","o","sub_i","path")],sub_path_df)
sub_path_df$classificationtime<-round(sub_path_df$classificationtime/1000000000,3)

sub_path_wide<-(reshape(sub_path_df, timevar = "sub_i", idvar = c("o","param_reasoner","path"), direction = "wide"))
sub_path_wide<-sub_path_wide[c("param_reasoner","o","path","classificationtime.1","classificationtime.2","classificationtime.3","classificationtime.4","classificationtime.5","classificationtime.6","classificationtime.7","classificationtime.8")]
names(sub_path_wide)<-c("r","o","path","p1","p2","p3","p4","p5","p6","p7","p8")
sub_path_wide<-sub_path_wide[order(sub_path_wide$o,sub_path_wide$r),]
sub_path_wide$id<-paste(sub_path_wide$r,sub_path_wide$o,sub_path_wide$path,sep="_")
x<-sub_path_wide
sub_path_wide$hardsubset<-ifelse(!is.na(x$p8)&(!is.na(x$p1)&x$p1>=x$p8)|(!is.na(x$p2)&x$p2>=x$p8)|(!is.na(x$p3)&x$p3>=x$p8)|(!is.na(x$p4)&x$p4>=x$p8)|(!is.na(x$p5)&x$p5>=x$p8)|(!is.na(x$p6)&x$p6>=x$p8)|(!is.na(x$p7)&x$p7>=x$p8),TRUE,FALSE)
sub_path_wide$tboxmeas<-ifelse(!is.na(sub_path_wide$p8),TRUE,FALSE)
path_df_anal<-analysepath(sub_path_wide,"id",c("p1","p2","p3","p4","p5","p6","p7","p8"))
sub_path_wide<-merge(sub_path_wide,path_df_anal, by="id", all.x = TRUE)
sub_path_wide$completeness<-ifelse(sub_path_wide$ct_nna_path==8,"complete",ifelse(sub_path_wide$ct_nna_path==0,"empty","partial"))

#### Relative Values for path ####
sub_path_wide_rel<-sub_path_wide[c("r","o","path")]
sub_path_wide_rel$p1<-sub_path_wide$p1/sub_path_wide$p8
sub_path_wide_rel$p2<-sub_path_wide$p2/sub_path_wide$p8
sub_path_wide_rel$p3<-sub_path_wide$p3/sub_path_wide$p8
sub_path_wide_rel$p4<-sub_path_wide$p4/sub_path_wide$p8
sub_path_wide_rel$p5<-sub_path_wide$p5/sub_path_wide$p8
sub_path_wide_rel$p6<-sub_path_wide$p6/sub_path_wide$p8
sub_path_wide_rel$p7<-sub_path_wide$p7/sub_path_wide$p8
sub_path_wide_rel$p8<-1

#### Create long version of interesting paths ####
sub_path_long<-melt(sub_path_wide[c("r","o","path","id","p1","p2","p3","p4","p5","p6","p7","p8")],id.vars = c("r","o","path","id"))
interestingpaths<-path_df_anal[!path_df_anal$monotonic,]$id
sub_path_long_interesting<-sub_path_long[sub_path_long$id %in% interestingpaths,]

## Prepare Modules ##
subset_modules<-read.csv(file=subset_modulesf,head=TRUE,sep=",")
subset_modules$filename<-gsub("obi_bcgo", "obi-bcgo", subset_modules$filename)
subset_modules$o<-getontnamefromlongname(subset_modules$filename,7,TRUE)
subset_modules$r<-subset_modules$param_reasoner
subset_modules<-subset_modules[subset_modules$o %in% unique(corpus_md$o),]
subset_modules_fail<-subset_modules[is.na(subset_modules$consistent),]
subset_modules<-subset_modules[!is.na(subset_modules$consistent),]
subset_modules<-merge(subset_modules,baseline[c("o","r","cltime_o","size_o")],by=c("o","r"),all.x=TRUE)
subset_modules$hardmodule<-ifelse(subset_modules$cltime_o<subset_modules$classificationtime,TRUE,FALSE)
subset_modules$sub_i<-getontnamefromlongname(subset_modules$filename,4,TRUE)
subset_modules$region<-paste(subset_modules$o,subset_modules$r,subset_modules$sub_i,sep="_")


#### Merge with O metadata ####
subset_modules<-merge(subset_modules,corpus_md[c("o","cat_main","o_consistent","cat_prof","sizebin")],by="o",all.x=TRUE)

mod<-subset_modules[c("filename","classificationtime","r","o","logical_axiom_count_incl")]
names(mod)<-c("mod","cl_mod","r","o","size_mod")
mod$hs<-gsub("^m_bottom_","",mod$mod)
mod<-mod[complete.cases(mod),]

#HS-MOD-O example PLOT
stab<-"C:/Dropbox/PHD/Thesis/thesis_final/graphics/example_stability.csv"
stability<-read.csv(file=stab,head=TRUE,sep=",")
ggplot(stability,aes(x=module,y=duration, group=group)) + geom_line() + theme_gray(base_size = 22) + xlab("") + ylab("Classification time")

### Modules Pathological Cases ###
subset_modules$cl_mod_sec<-subset_modules$classificationtime/1000000000
subset_modules$cl_o_sec<-subset_modules$cltime_o/1000000000
subset_modules$size_diff_rel<-round(100*(subset_modules$logical_axiom_count_incl/subset_modules$size_o),2)
subset_modules$clt_diff<-subset_modules$cl_o_sec-subset_modules$cl_mod_sec
subset_modules$subsec<-ifelse(abs(subset_modules$clt_diff)<1,TRUE,FALSE)
subset_modules$clt_diff_fchange <- mapply(foldChangeNormalised,subset_modules$cl_mod_sec,subset_modules$cl_o_sec)
subset_modules$clt_diff_fchange_norm <- mapply(illegalNormalisation,subset_modules$clt_diff_fchange)
subset_modules$significant_difference<- ifelse(abs(subset_modules$clt_diff_fchange_norm)>=0.05,TRUE,FALSE)


m_patho<-subset_modules[subset_modules$hardmodule&subset_modules$cat_main!="OWL Full"&!is.na(subset_modules$classificationtime),]
nrow(m_patho[!m_patho$subsec,])

### Modules pathological with hard subset ontology measurements ###
m_patho_hardsubmeas<-merge(m_patho[c("o","r","filename","logical_axiom_count_incl","classificationtime","cat_main")],subset_o[c("o","r","logical_axiom_count_incl","cltime_o")],by=c("o","r"))

nrow(m_patho_hardsubmeas)
m_patho_hardsubmeas$hardmodule<-ifelse(m_patho_hardsubmeas$cltime_o<m_patho_hardsubmeas$classificationtime,TRUE,FALSE)
nrow(m_patho_hardsubmeas)
m_patho_hardsubmeas<-m_patho_hardsubmeas[m_patho_hardsubmeas$cat_main!="OWL Full",]
nrow(m_patho_hardsubmeas)



#### Hard subsets: Overarching frame merge, merge_sub ####
hs<-subset_all_hard[c("filename","classificationtime","r","o","logical_axiom_count_incl")]
hs<-hs[complete.cases(hs),]
names(hs)<-c("hs","cl_hs","r","o","size_hs")

o<-subset_o[c("classificationtime","r","o","logical_axiom_count_incl")]
o<-o[complete.cases(o),]
names(o)<-c("cl_o","r","o","size_o")

merge<-merge(hs,mod,by=c("o","r","hs"),all=TRUE)
merge<-merge(merge,o,by=c("o","r"),all.x=TRUE)
merge$id<-1:nrow(merge)
m<-t(apply(merge[c("cl_o","cl_mod","cl_hs")], 1, function(x)(x-min(x))/(max(x)-min(x))))
m <- as.data.frame(m)
merge$cl_mod_n<-m$cl_mod
merge$cl_o_n<-m$cl_o
merge$cl_hs_n<-m$cl_hs
merge$region<-getontnamefromlongname(merge$hs,2,TRUE)
merge$om_diff<-merge$cl_o-merge$cl_mod
merge$om_diff_sec<-round(merge$om_diff/1000000000,2)
merge$om_diff_size<-merge$size_o-merge$size_mod
merge$om_diff_rel<-round(100*(merge$om_diff/merge$cl_o),1)
merge$om_diff_size_rel<-round(100*(merge$om_diff_size/merge$size_o),1)
merge$hardmodule<-ifelse(merge$cl_o<merge$cl_mod,TRUE,FALSE)
merge$hardsubset<-ifelse(merge$cl_o<merge$cl_hs,TRUE,FALSE)

x<-merge
merge$order<-ifelse((x$cl_o>x$cl_hs)&(x$cl_o>x$cl_mod)&(x$cl_hs>x$cl_mod),"OHM",
                    ifelse((x$cl_o>x$cl_hs)&(x$cl_o>x$cl_mod)&(x$cl_hs<x$cl_mod),"OMH",
                           ifelse((x$cl_o>x$cl_hs)&(x$cl_o<x$cl_mod)&(x$cl_hs<x$cl_mod),"MOH",
                                  ifelse((x$cl_o<x$cl_hs)&(x$cl_o<x$cl_mod)&(x$cl_hs<x$cl_mod),"MHO",
                                         ifelse((x$cl_o<x$cl_hs)&(x$cl_o>x$cl_mod)&(x$cl_hs>x$cl_mod),"HOM",
                                                ifelse((x$cl_o<x$cl_hs)&(x$cl_o<x$cl_mod)&(x$cl_hs>x$cl_mod),"HMO",NA))))))

merge_sub<-merge[c("id","o","r","hs","mod","cl_hs_n","cl_mod_n","cl_o_n","order")]
merge_sub<-merge(merge_sub,corpus_md[c("o","cat_main","o_consistent","cat_prof","sizebin")],by="o",all.x=TRUE)
#merge_sub<-merge_sub[sample(nrow(merge_sub), 1000), ]
merge_sub<-merge_sub[!is.na(merge_sub$order),]
#### Exlude cases where HS < O
merge_sub<-merge_sub[merge_sub$cl_hs_n>merge_sub$cl_o_n,]
merge_sub$region<-getontnamefromlongname(merge_sub$hs,2,TRUE)

df_hom<-merge[!is.na(merge$order),]
df_hom<-merge(df_hom,corpus_md[c("o","expressivity_incl.x","cat_main")],by=c("o"),all.x=TRUE)
#df_hom<-df_hom[df_hom$cl_hs_n>df_hom$cl_o_n,]
df_hom$region<-getontnamefromlongname(df_hom$hs,2,TRUE)
df_hom$cl_o_sec<-df_hom$cl_o/1000000000
df_hom$cl_mod_sec<-df_hom$cl_mod/1000000000
df_hom$cl_hs_sec<-df_hom$cl_hs/1000000000

df_hom$clt_diff_fchange <- mapply(foldChangeNormalised,df_hom$cl_mod_sec,df_hom$cl_o_sec)
df_hom$clt_diff_fchange_norm <- mapply(illegalNormalisation,df_hom$clt_diff_fchange)

df_hom$clt_hs_diff_fchange <- mapply(foldChangeNormalised,df_hom$cl_hs_sec,df_hom$cl_mod_sec)
df_hom$clt_hs_diff_fchange_norm <- mapply(illegalNormalisation,df_hom$clt_hs_diff_fchange)
df_hom$significant_difference<- ifelse(abs(df_hom$clt_diff_fchange_norm)>=0.05,TRUE,FALSE)

## POTECTIVE EFFECT

df_protective<-unique(df_hom[c("o","hs","r","mod","cat_main","cl_mod_sec","cl_hs_sec","clt_hs_diff_fchange_norm")])
ggplot(df_protective[df_protective$cat_main!="OWL Full",],aes(clt_hs_diff_fchange_norm,fill=o)) + geom_histogram() + facet_wrap("r") + xlab("Normalised fold change")
#### ANALYSSE df_hom for cross comparison


df_hom_patho<-df_hom[df_hom$hardmodule,]

final_hard_regions<-unique(merge_sub[merge_sub$cl_o_n<merge_sub$cl_mod_n,][c("o","r","region")])

## Real Modules: realm ##
realmods<-read.csv(file=realmodulesf,head=TRUE,sep=",")
realmods$filename<-gsub("obi_bcgo", "obi-bcgo", realmods$filename)
realmods$o<-get_ontology_short_name(realmods$filename)
realmods$o<-gsub("moremodule_(el|owl)_","",realmods$o)
realmods$decomposition<-ifelse(grepl("max_atom", realmods$filename),"chainsaw",ifelse(grepl("moremodule_el", realmods$filename),"more_el",ifelse(grepl("moremodule_owl", realmods$filename),"more_owl",NA)))
realmods$r<-realmods$param_reasoner
realmods<-realmods[realmods$o %in% unique(corpus_md$o),]
realmods_fail<-realmods[is.na(realmods$consistent),]
realmods<-realmods[!is.na(realmods$consistent),]
realmods<-merge(realmods,baseline[c("o","r","cltime_o","size_o")],by=c("o","r"),all.x=TRUE)
realmods$hardmodule<-ifelse(realmods$cltime_o<realmods$classificationtime,TRUE,FALSE)

#### Merge with O metadata ####
realmods<-merge(realmods,corpus_md[c("o","cat_main","o_consistent","cat_prof","sizebin")],by="o",all.x=TRUE)
realmods$om_diff_size<-realmods$size_o-realmods$logical_axiom_count_incl

#### Filter out modules for which we do not have a O time
realm<-realmods[!is.na(realmods$cltime_o)&realmods$om_diff_size>0,]
#stab<-"D:/Dropbox/PHD/Thesis/thesis_final/graphics/example_stability.csv"
realm$size_diff_rel<-round(100*(realm$logical_axiom_count_incl/realm$size_o),2)
realm$cl_mod_sec<-realm$classificationtime/1000000000
realm$cl_o_sec<-realm$cltime_o/1000000000
realm$clt_diff<-realm$cl_o_sec-realm$cl_mod_sec
realm$clt_rel<-realm$cl_mod_sec/realm$cl_o_sec
realm$clt_diff_fchange <- mapply(foldChangeNormalised,realm$cl_mod_sec,realm$cl_o_sec)
realm$clt_diff_fchange_norm <- mapply(illegalNormalisation,realm$clt_diff_fchange)
realm$om_diff_size_rel<-round(100*(realm$logical_axiom_count_incl/realm$size_o),1)

realm$significant_difference<- ifelse(abs(realm$clt_diff_fchange_norm)>=0.05,TRUE,FALSE)
realm$clt_diff_fchange_norm_pc<-round(realm$clt_diff_fchange_norm*100,2)
realm$clt_diff_rel<-realm$clt_diff/realm$cl_o_sec
realm$subsec<-ifelse(abs(realm$clt_diff)<1,TRUE,FALSE)
realm$rel_size_category<-ifelse(realm$size_diff_rel>90,"almost o",ifelse(realm$size_diff_rel>75,"very large",ifelse(realm$size_diff_rel>50,"large",ifelse(realm$size_diff_rel>25,"medium",ifelse(realm$size_diff_rel>10,"small","very small")))))


count_patho_all<-count(realm[realm$cat_main!="OWL Full"&realm$significant_difference&realm$hardmodule&!realm$subsec,][c("decomposition")])
count_patho_more<-count(realm[realm$decomposition=="more_owl"&realm$hardmodule&realm$cat_main!="OWL Full"&realm$significant_difference,][c("decomposition","o","r")])
count_patho_chainsaw<-count(realm[realm$decomposition=="chainsaw"&realm$hardmodule&realm$cat_main!="OWL Full"&realm$significant_difference,][c("decomposition","o","r")])
count_os_with_pcases<-count(unique(realm[realm$hardmodule&realm$significant_difference,][c("r","o","decomposition")])[c("o","decomposition")])
count_size_differences<-count(realm[realm$cat_main!="OWL Full"&realm$hardmodule&realm$significant_difference,][c("rel_size_category")])
xx<-sum(count_size_differences$freq)
count_size_differences$rel<-round((count_size_differences$freq/xx)*100,2)

count_o_gt_10sec<-count(realm[realm$cat_main!="OWL Full"&realm$cl_o_sec>=10,][c("hardmodule","significant_difference")])
count_o_gt_10sec$rel<-round((count_o_gt_10sec$freq/sum(count_o_gt_10sec$freq))*100,2)
count_o_st_10sec<-count(realm[realm$cat_main!="OWL Full"&realm$cl_o_sec<10,][c("hardmodule","significant_difference")])
count_o_st_10sec$rel<-round((count_o_st_10sec$freq/sum(count_o_st_10sec$freq))*100,2)

count_o_gt_10sec
count_o_st_10sec

#### Pathological modules: realm_patho ####
realm_patho<-realm[realm$hardmodule,][c("filename","o","r","cat_main","size_o","logical_axiom_count_incl","significant_difference","size_diff_rel","classificationtime","hardmodule","cltime_o","subsec")]

realm_patho<-realm_patho[realm_patho$cat_main!="OWL Full",]
nrow(realm_patho[realm_patho$significant_difference,])

#ggplot(stability,aes(x=module,y=duration,colour=run)) + geom_line()

# Analysis #
nrow(subset_path)
nrow(subset_path_fail)
nrow(subset_all)
nrow(subset_all_fail)
nrow(subset_modules)
nrow(subset_modules_fail)

## PATH ANALYSIS ##
total_subsets<-6888
totalo<-328
nfailures<-(total_subsets*3)-nrow(subset_path)
npaths<-total_subsets/7
print(round(nrow(subset_path)/(total_subsets*3),2))

path_comp_by_r<-count(subset_path[c("param_reasoner")])
path_comp_by_r$percent<-round(path_comp_by_r$freq/total_subsets,2)

path_comp_fail_by_r<-count(subset_path_fail[c("exception")])
path_comp_fail_by_r$percent<-round(path_comp_fail_by_r$freq/nfailures,2)

o_comp_by_r<-count(baseline[c("param_reasoner")])
o_comp_by_r$percent<-round(o_comp_by_r$freq/totalo,2)

o_comp_fail_by_r<-count(baseline_fail[c("exception")])
o_comp_fail_by_r$percent<-round(o_comp_fail_by_r$freq/((totalo*3)-nrow(baseline)),2)

path_completeness_by_reasoner<-count(sub_path_wide[c("r","completeness","tboxmeas")])
path_completeness_by_reasoner$rel<-round(path_completeness_by_reasoner$freq/984,2)

p.plot<-ggplot(sub_path_long_interesting,aes(x=variable,y=value,group=id,colour=r)) + geom_line()
p.plot<-p.plot + facet_wrap("o", scales = "free")
#p.plot + facet_grid(r~o, scales = "free")

### Hard regions ####
#count_hr<-count(regions[c("r","tooeasy","hardregion")])

nrow(subset_path)

## HARD SUBSET ANALYSIS ##
#subset_all

#subset_all_hard
#subset_modules
#subset_o
agg_subset_all<-aggregate(classificationtime ~ filename+r, data = subset_all, FUN = function(x) c(mean=mean(x), max=max(x),min=min(x)))
agg_subset_all$o<-getontnamefromlongname(agg_subset_all$filename,5,TRUE)
agg_subset_all<-merge(agg_subset_all,baseline[c("o","cltime_o","r")],by=c("o","r"),all.x=TRUE)
agg_subset_all <- do.call(data.frame, agg_subset_all)
names(agg_subset_all)<-c("o","r","subset","clt_mean","clt_max","clt_min","clt_o")
agg_subset_all$hard<-ifelse(agg_subset_all$clt_o<agg_subset_all$clt_min,"hard_min",ifelse(agg_subset_all$clt_o<agg_subset_all$clt_mean,"hard_mean",ifelse(agg_subset_all$clt_o<agg_subset_all$clt_max,"hard_max","nothard")))

agg_subset_o<-aggregate(classificationtime ~ filename, data = subset_o, FUN = function(x) c(mean=mean(x), max=max(x),min=min(x)))
agg_subset_o$ct_o_mean<-agg_subset_o$classificationtime.mean
agg_subset_o$ct_o_min<-agg_subset_o$classificationtime.min
agg_subset_o$ct_o_max<-agg_subset_o$classificationtime.max

#merge_sub<-merge_sub[merge_sub$cat_main!="OWL Full",]

merge_long<-melt(merge_sub,id.vars=c("id","o","r","hs","mod","order","cat_main","o_consistent","cat_prof","sizebin"))
p.hom<-ggplot(merge_long,aes(x=variable,y=value,group=id,colour=r)) + geom_line()
p.hom1<-p.hom + facet_grid(order~cat_main)
p.hom2<-p.hom + facet_grid(o~order)

p.order<-ggplot(merge_sub,aes(x=order)) + geom_histogram()
p.order1<- p.order + facet_grid(r~cat_main) + scale_y_log10()
p.order2<- p.order + facet_grid(o~r) + scale_y_log10()

o_with_hs<-unique(subset_modules[subset_modules$hardmodule,]$o)
o_in_full<-unique(subset_modules[subset_modules$cat_main=="OWL Full",]$o)
p.path_hm<-ggplot(sub_path_long[sub_path_long$o %in% o_with_hs&!(sub_path_long$o %in% o_in_full),],aes(x=variable,y=value,group=id,colour=r)) + geom_line()
p.path_hm<-p.path_hm + facet_wrap("o", scales = "free") + xlab("Region") + ylab("Classification time (sec)")

p.path_hmfull<-ggplot(sub_path_long[sub_path_long$o %in% o_with_hs&(sub_path_long$o %in% o_in_full),],aes(x=variable,y=value,group=id,colour=r)) + geom_line()
p.path_hmfull<-p.path_hmfull + facet_wrap("o", scales = "free")

count_hmo_cases<-count(merge_sub[c("order")])
count_hmo_cases$rel<-round(count_hmo_cases$freq/nrow(merge_sub),2)

count_modules_nfull_hard<-count(subset_modules[subset_modules$cat_main!="OWL Full"&subset_modules$hardmodule,][c("o","r")])

## Real Modules ##
final_hard_regions<-unique(merge_sub[merge_sub$cl_o_n<merge_sub$cl_mod_n,][c("o","r","region")])
p.fhardreg<-ggplot(final_hard_regions,aes(x=region,fill=r))+geom_histogram(position = "stack")

final_hard_subsets_by_region<-unique(merge_sub[merge_sub$cl_o_n<merge_sub$cl_mod_n,][c("mod","o","r","region")])
p.fhardsub<-ggplot(final_hard_subsets_by_region,aes(x=region,fill=r))+geom_histogram(position = "stack") + facet_wrap("o")
### PUT THE FOLLOWING
p.hinodifftime<-ggplot(merge[merge$o=="hino"&merge$cl_o<merge$cl_mod,],aes(x=abs(om_diff_sec))) + geom_histogram()
p.difftime<-ggplot(merge[merge$cl_o<merge$cl_mod,],aes(x=abs(om_diff_sec),fill=r)) + geom_histogram() + scale_x_log10(breaks=c(0.1,1,10,50)) 

#### This plot shows nicely that the difference in size does account for the difference in perfroamnce
p.size_to_diff<-ggplot(merge,aes(x=om_diff_size_rel,y=om_diff_rel,colour=hardmodule)) + geom_point() + facet_wrap("o")

#### The same cannot be said for the pathological ones though ####
p.size_to_diff_patho<-ggplot(merge[merge$hardmodule,],aes(x=om_diff_size_rel,y=om_diff_rel,colour=hardmodule,shape=r)) + geom_point() + facet_wrap("o")


### Compare pathological vs real modules ###
m1<-m_patho[c("size_diff_rel","subsec")]
m2<-realm_patho[realm_patho$hardmodule,][c("size_diff_rel","subsec")]
m3<-realm[c("size_diff_rel","subsec")]
m4<-subset_modules[c("size_diff_rel","subsec")]
m1$group<-"Hard subset modules (Pathological)"
m2$group<-"Real modules (Pathological)"
m3$group<-"Real modules (All)"
m4$group<-"Hard subset modules (All)"
m1$Type<-"Pathological"
m2$Type<-"Pathological"
m3$Type<-"All"
m4$Type<-"All"
m1$g1<-"HS"
m2$g1<-"Real"
m3$g1<-"Real"
m4$g1<-"HS"
names(m1)<-c("size","subsec","group","Type","g1")
names(m2)<-c("size","subsec","group","Type","g1")
names(m3)<-c("size","subsec","group","Type","g1")
names(m4)<-c("size","subsec","group","Type","g1")
df_patho_plot_dist<-rbind(m1,m2,m3,m4)

ggplot(df_patho_plot_dist[df_patho_plot_dist$g1=="HS",],aes(x=size,binwidth=1000,fill=Type)) + geom_histogram(position="dodge") + scale_y_log10() + xlab("Number of axioms in % of |O|") + theme_gray(base_size = 20)
ggplot(df_patho_plot_dist[df_patho_plot_dist$g1=="Real",],aes(x=size,binwidth=1000,fill=Type)) + geom_histogram(position="dodge") + scale_y_log10() + xlab("Number of axioms in % of |O|") + theme_gray(base_size = 20)

## Variance analysis
covar_mod<-aggregate(merge$cl_mod, by=list(merge$mod,merge$r), FUN=co.var)
covar_mod$pc<-covar_mod$x*100
covar_hs<-aggregate(merge$cl_hs, by=list(merge$hs,merge$r), FUN=co.var)
covar_hs$pc<-covar_hs$x*100
covar_o<-aggregate(merge$cl_o, by=list(merge$o,merge$r), FUN=co.var)
covar_o$pc<-covar_o$x*100
covar_mod<-merge(covar_mod,unique(merge[c("mod","size_mod")]),all.x=TRUE,by.x="Group.1",by.y="mod")
covar_mod$type<-"module"
covar_hs<-merge(covar_hs,unique(merge[c("hs","size_hs")]),all.x=TRUE,by.x="Group.1",by.y="hs")
covar_hs$type<-"hs"
covar_o<-merge(covar_o,unique(merge[c("o","size_o")]),all.x=TRUE,by.x="Group.1",by.y="o")
covar_o$type<-"o"
names(covar_o)<-c("o","r","cov","covp","size","type")
names(covar_mod)<-c("o","r","cov","covp","size","type")
names(covar_hs)<-c("o","r","cov","covp","size","type")
covar_all<-rbind(covar_o[c("o","r","covp","size","type")],covar_mod[c("o","r","covp","size","type")],covar_hs[c("o","r","covp","size","type")])
summary(covar_mod$pc)
summary(covar_hs$pc)
summary(covar_o$pc)

ggplot(covar_all,aes(x=covp)) + geom_histogram() + facet_wrap("type", scales="free_y") + xlab("Coefficient of variation (COV)")+ theme_gray(base_size = 20)

round((nrow(covar_all[covar_all$covp>5,])/nrow(covar_all))*100,2)
