corpusmetrics_character <- c("filepath" ,"exception" ,"param_ontfile" ,"experiment_id" ,"filename" ,"experiment_runid" ,"exception_message" ,"jar_name" ,"experiment_version" ,"experiment_date" ,"owl2dl_profile_violation" ,"axiom_types"  ,"axiom_types_incl"  ,"valid_imports"  ,"owlapi_version" ,"constructs_incl"   ,"constructs"  ,"stripped_axiomtypes"  ,"expressivity"  ,"axiom_type_count_incl"  ,"dt_notbuiltin_ct_incl"   ,"fixed_profile_violations"  ,"expressivity_incl" ,"exception_cause")
phasemetrics_nanoseconds <- c("A1reason_pp_dur","A2reason_cc_dur","A3reason_pro_dur","A4reason_trav_dur","A5reason_pop_dur","B1ontologyloadtime","B2createreasonertime","B3classificationtime","B4disposereasonertime","M1reason_pp_dur","M2reason_mmcl_dur","M3reason_pop_dur","M91reason_dec_dur","M92reason_cc_dur","cltime_all","cltime_reasoning","cmp_mon_cltime_all","cmp_mon_cltime_reasoning","cmp_mon_cltime_st","cmp_mon_cltime_subtests","delreasoner_worst_cltime","tt_del_cc","tt_del_pop","tt_del_pp","tt_del_prp","tt_del_st","tt_sat_red_bc","tt_sat_red_first","tt_sat_red_wc","tt_sat_unique_bc","tt_sat_unique_first","tt_sat_unique_wc")
phasemetrics_character <- c("run_group","delreasoner", "delreasoner_cl","delreasoner_version", "exception", "exception_message", "experiment_date", "experiment_id", "experiment_runid", "experiment_version", "filename", "filepath","jar_name","param_checkcorrectness", "param_gathersat", "param_ontfile", "param_outdir", "param_reasoner", "param_reasonertimeout","reasoner", "reasoner_class", "reasoner_version", "reasonerid")
mod_reasoner_phases <- c("M1reason_pp_dur",  "M2reason_mmcl_dur",  "M3reason_pop_dur")
mon_reasoner_phases <- c("A1reason_pp_dur","A2reason_cc_dur",  "A3reason_pro_dur",  "A4reason_trav_dur",	"A5reason_pop_dur")
owlapiphases <- c("B1ontologyloadtime","B2createreasonertime","B3classificationtime",  "B4disposereasonertime")
mono <- c("Pellet[exp]","HermiT[exp]","JFact[exp]")

scoreTechnique<-function(df_rank_in) {
  df_rank_in_long<-melt(df_rank_in,id.vars=c("o","primarydelegate","technique","el"))
  df_rank_in_long$model<-ifelse(grepl("^bb_",df_rank_in_long$variable),"BB",ifelse(grepl("^gb_",df_rank_in_long$variable),"GB",ifelse(grepl("^ec_",df_rank_in_long$variable),"EC",ifelse(grepl("^ecgb_",df_rank_in_long$variable),"ECGB",NA))))
  df_rank_in_long<-df_rank_in_long[order(df_rank_in_long$value),]
  df_ranked<-transform(df_rank_in_long, rank = ave(value, o, primarydelegate, model, FUN = function(x) rank(x, ties.method = "first")))
  df_ranked$points<-ifelse(df_ranked$rank==1,5,ifelse(df_ranked$rank==2,4,ifelse(df_ranked$rank==3,3,ifelse(df_ranked$rank==4,2,ifelse(df_ranked$rank==5,1,0)))))
  df_ranked$tech_el<-paste(df_ranked$technique," (",df_ranked$el,")",sep="")
  return(df_ranked)
}

rankTechnique<-function(df_scored) {
  df_toplist<-aggregate(points ~ primarydelegate+model+technique+el+tech_el, data = df_scored, FUN = sum)
  return(df_toplist)
}

analysedata <- function(data) {
  #phases_all <- c(mod_reasoner_phases,mon_reasoner_phases)
  #phases_reasoning <- c("M2reason_mmcl_dur","A2reason_cc_dur", "A3reason_pro_dur", "A4reason_trav_dur")
  #data$cltime_all <- apply(data[,phases_all], 1, function(x) { sum(x,na.rm = TRUE) } )
  #data$cltime_reasoning <- apply(data[,phases_reasoning], 1, function(x) { sum(x,na.rm = TRUE) } )
  
  #Performance
  
  data$bb_oct<-data$cltime_all-data$M92reason_cc_dur
  data$bb_oct2<-data$M1reason_pp_dur+data$M2reason_mmcl_dur+data$M3reason_pop_dur
  data$bb_sst<-data$tt_sat_all
  data$bb_sst_red<-data$tt_sat_red_wc
  data$bb_sst_un<-data$tt_sat_unique_wc
  data$bb_scc<-data$tt_del_cc
  data$bb_dec<-data$M91reason_dec_dur
  data$bb_cst<-data$sat_ct_total
  data$bb_prp<-data$tt_del_prp
  data$bb_pp<-data$tt_del_pp
  data$bb_overhead<-data$overhead_time
  data$bb_ppmod<-data$M1reason_pp_dur-data$M92reason_cc_dur-data$M91reason_dec_dur
  
  data$bb_oct_wo_rest<-data$bb_oct-data$bb_pp-data$bb_scc-data$bb_prp-data$bb_sst-data$bb_dec-data$bb_ppmod-data$overhead_time
  
  data$ec_oct<-data$bb_oct-data$M91reason_dec_dur
  
  data$gb_oct<-data$bb_oct-data$tt_sat_red_wc-data$tt_del_cc
  data$gb_sst<-data$tt_sat_all-data$tt_sat_red_wc
  data$gb_cst<-data$sat_ct_unique
  
  data$ecgb_oct<-data$gb_oct-data$M91reason_dec_dur
  
  
  #Overhead
  #data$bb_overhead<-data$M91reason_dec_dur
  
  
  #Redundancy
  data$red_test_ct<-data$sat_ct_red
  data$rat_red_test_ct_all<-val_pc(data$red_test_ct,data$sat_ct_total)
  data$bb_red_test_tt<-data$tt_sat_red_bc
  
  #Test Hardness
  
  data$rat_delreasoner_sat_nosat <- (data$delreasoner_ct - data$delreasoner_conductedsat_ct)/((data$delreasoner_ct + data$delreasoner_conductedsat_ct)/2)
  data$rat_delreasoner_sat_all <- (data$delreasoner_conductedsat_ct/data$delreasoner_ct)
  
  data$pp_dec_pdiff <- (data$M1reason_pp_dur - data$M91reason_dec_dur)/((data$M1reason_pp_dur + data$M91reason_dec_dur)/2)
  #data$cmp_rat_redavoid_total_sat <- (data$diff_sat_ct_avoided_redundant/data$delreasoner_ct)
  #data$cmp_sattimepure_fchange <- mapply(foldChangeNormalised,data$cmp_mon_cltime_subtests,data$cltime_subtests)
  #data$cmp_sattimepure_first_fchange <- mapply(foldChangeNormalised,data$cmp_mon_cltime_subtests,data$tt_sat_unique_first)
  #data$cmp_sattimepure_wc_fchange <- mapply(foldChangeNormalised,data$cmp_mon_cltime_subtests,data$tt_sat_unique_wc)
  #data$cmp_wo_dec_fchange <- mapply(foldChangeNormalised,data$cmp_mon_cltime_all,data$cltime_wo_dec)
  
  data$rat_red_sat_tt_cltime_all <- val_pc(data$tt_sat_red_bc,data$bb_oct) 
  
  ## contributors to OCT
  data$rat_m1pp_oct <- val_pc(data$M1reason_pp_dur-data$M92reason_cc_dur,data$bb_oct)
  data$rat_m1ppwoccdec_oct <- val_pc((data$M1reason_pp_dur-data$M91reason_dec_dur-data$M92reason_cc_dur),data$bb_oct)
  data$rat_m91dec_oct <- val_pc(data$M91reason_dec_dur,data$bb_oct)
  
  data$rat_m2modcl_oct <- val_pc(data$M2reason_mmcl_dur,data$bb_oct)
  data$rat_m3pop_oct <- val_pc(data$M3reason_pop_dur,data$bb_oct)
  
  data$rat_cc_del_oct <- val_pc(data$tt_del_cc,data$bb_oct)
  data$rat_pp_del_oct <- val_pc(data$tt_del_pp,data$bb_oct)
  data$rat_st_del_oct <- val_pc(data$tt_del_st,data$bb_oct)
  data$rat_prp_del_oct <- val_pc(data$tt_del_prp,data$bb_oct)
  data$rat_pop_del_oct <- val_pc(data$tt_del_pop,data$bb_oct)
  
  data$rat_m2modcl_rest_oct <- val_pc(data$M2reason_mmcl_dur-data$tt_del_pp-data$tt_del_cc-data$tt_del_prp-data$tt_del_st-data$tt_del_pop,data$bb_oct) 
  
  data$rat_worst_del_oct <- val_pc(data$delreasoner_worst_cltime,data$bb_oct)
  
  data$delreasoner_ct<-data$owl_delreasoner_ct+data$el_delreasoner_ct
  secs<-names(data[ , grepl( "^(bb|gb|ec|ecgb)_" , names( data ) ) ])
  data<-getdatainseconds(data,secs)
  return(data)
}

plotphase <- function(data) {
  exportplots(plotclassificationphases(data,mean,subsetmorethanonesecond))
  exportplots(plotclassificationphases(data,median,subsetmorethanonesecond))
  exportplots(plotclassificationphases_mod(data,mean,subsetmorethanonesecond))
  exportplots(plotclassificationphases_mod(data,median,subsetmorethanonesecond))
  exportplots(plotowlapiphases(data,mean,subsetmorethanonesecond))
  exportplots(plotowlapiphases(data,median,subsetmorethanonesecond))
  exportplots(plotclassificationphases(data,mean,subsetlessthanonesecond))
  exportplots(plotclassificationphases(data,median,subsetlessthanonesecond))
  exportplots(plotclassificationphases_mod(data,mean,subsetlessthanonesecond))
  exportplots(plotclassificationphases_mod(data,median,subsetlessthanonesecond))  
  exportplots(plotowlapiphases(data,median,subsetlessthanonesecond))
  exportplots(plotowlapiphases(data,mean,subsetlessthanonesecond))
  data_all <- mergeonfilename(data$phasedata,data$corpus)
  modular_data <- subset(data_all, !(data_all$reasoner %in% mono))
  atleasttwoseconds <- subset(modular_data,modular_data$cltime_all>2)
  
  cool <- c("tboxrbox_size_incl","class_count_incl","logical_axiom_count_incl","ax_complexrhs_ct","cltime_all","cltime_reasoning")
  exportplots(plotcorrelation(data$phasedata,owlapiphases,"corr_owlapimetrics"))
  exportplots(plotcorrelation(data$phasedata,mon_reasoner_phases,"corr_ctimes_mono"))
  exportplots(plotcorrelation(data$phasedata,mod_reasoner_phases,"corr_ctimes_mod"))
  #exportplots(plotcorrelation(data$phasedata,phasemetrics_nanoseconds,"corr_all_phase"))
  exportplots(plotcorrelation(modular_data,cool,"rt_ont_corr"))
  #exportplots(plotmultiplot(modular_data,"diff_sat_ct_harder_easier_overall"))
  #exportplots(plotsinglevariablelinegraphs(modular_data,"rat_dec_cltime_all"))
  #exportplots(plotsinglevariablelinegraphs(modular_data,"rat_delr_sat"))
  #exportplots(plotsinglevariablelinegraphs(modular_data,"rat_tt_sat_st"))
  exportplots(boxplotmultiplecolumns_logy(data_all,mon_reasoner_phases,"boxplot_mon_reasoner_phases"))
  exportplots(boxplotmultiplecolumns_logy(data_all,mod_reasoner_phases,"boxplot_mod_reasoner_phases"))
  exportplots(boxplotmultiplecolumns_logy(data_all,owlapiphases,"boxplot_owlapi_reasoner_phases"))
  exportplots(plotmultiplot(modular_data,"rat_dec_cltime_all"))
  exportplots(plotmultiplot(modular_data,"rat_pp_dec"))
  exportplots(plotmultiplot(modular_data,"rat_tt_sat_st"))
  exportplots(plotmultiplot(modular_data,"rat_st_del_cltime_all"))
  exportplots(plotmultiplot(modular_data,"rat_cc_del_cltime_all"))
  exportplots(plotmultiplot(modular_data,"rat_prp_del_cltime_all"))
  exportplots(plotmultiplot(modular_data,"rat_pp_del_cltime_all"))
  exportplots(boxplotmultiplecolumns(modular_data,c("rat_dec_cltime_all","rat_pp_dec","rat_tt_sat_st","rat_pp_del_cltime_all","rat_cc_del_cltime_all","rat_prp_del_cltime_all","rat_st_del_cltime_all"),"boxplot_someratiodata"))
  
  exportplots(plotcltimeeffect(modular_data,"cmp_cltime_all_fchange","Overall Cl-time, fold change, all cases"))
  exportplots(plotcltimeeffect(modular_data,"cmp_wo_dec_fchange","Overall Cl-time excluding decomposition, fold change, all cases"))
  exportplots(plotcltimeeffect(modular_data,"cmp_sattimepure_fchange","Pure Subsumption-test time, fold change"))
  exportplots(plotcltimeeffect(modular_data,"cmp_sattimepure_first_fchange","Pure Subsumption-test time, first test, fold change"))
  exportplots(plotcltimeeffect(modular_data,"cmp_sattimepure_wc_fchange","Pure Subsumption-test time, worst test, fold change"))
  
  exportplots(plotcltimeeffect(atleasttwoseconds,"cmp_cltime_all_fchange","Overall Cl-time, fold change, cltime gt 2 sec"))
  exportplots(plotcltimeeffect(atleasttwoseconds,"cmp_wo_dec_fchange","Overall Cl-time excluding decomposition, fold change, cltime gt 2 sec"))
  exportplots(plotcltimeeffect(atleasttwoseconds,"cmp_sattimepure_fchange","Pure Subsumption-test time, fold change, cltime gt 2 sec"))
  exportplots(plotcltimeeffect(atleasttwoseconds,"cmp_sattimepure_first_fchange","Pure Subsumption-test time, first test, fold change, cltime gt 2 sec"))
  exportplots(plotcltimeeffect(atleasttwoseconds,"cmp_sattimepure_wc_fchange","Pure Subsumption-test time, worst test, fold change, cltime gt 2 sec"))
  exportplots(plotsattesthardness(atleasttwoseconds,"Subsumption test hardness"))
  
  interestingontologymetrics <- c("abox_size_incl" ,"ax_complexrhs_ct_incl" ,"tboxrbox_size_incl" ,"logical_axiom_count" ,"tboxrbox_size" ,"avg_size_complexrhs" ,"tbox_size_incl_before","dt_notbuiltin_ct","avg_size_complexrhs_incl" ,"obj_property_count_incl" ,"ax_complexrhs_ct","class_count" ,"indiv_count_incl","indiv_count" ,"dataproperty_count_incl" ,"class_count_incl" ,"dataproperty_count" ,"owl2" ,"rdfs" ,"obj_property_count" ,"owl2_ql" ,"logical_axiom_count_incl"  ,"owl2_rl","rule_ct_incl","datatypes_count" ,"dt_builtin_ct","signature_size" ,"datatypes_count_incl" ,"rbox_size_incl","tbox_size_incl" ,"tbox_size" ,"rbox_size","rule_ct"  ,"signature_size_incl" ,"owl2_dl" ,"abox_size","owl2_el")
  interestingphasemetrics <- c("cltime_all","cmp_cltime_all_fchange","cmp_sattimepure_fchange")
  for(om in interestingontologymetrics) {
    for(pm in interestingphasemetrics) {
      print(paste("Plotting: ",om," and ",pm,sep=""))
      exportplots(plotcorrelationmultiplot(modular_data,pm,om))
    }
  }
}

loadphasedata <- function(filename) {
  data <- read.csv(paste("data/",filename,sep=""),colClasses = "character")
  x <- names(data)
  phasemetrics_numeric <- x[!x %in% phasemetrics_character]
  data[, phasemetrics_numeric] <- sapply(data[, phasemetrics_numeric], as.numeric)
  return(data)
}

#Filters
subsetmorethanonesecond <- function(data) {
  return(data[data$B3classificationtime >= 1, ])
}
subsetlessthanonesecond <- function(data) {
  return(data[data$B3classificationtime <= 1, ])
}
subsetinteresting <- function(data) {
  ontologies <- unique(data$filename)
  monolithicdata <- subset(data, data$reasoner %in% mono)
  interesting <- c()
  uninteresting <- c()
  for(ont in ontologies) {
    subsetont  <- subset(monolithicdata, monolithicdata$filename==ont)
    subsetsat  <- subset(subsetont,subsetont$sat_ct_total>0)
    if(nrow(subsetsat)>0) {
      interesting <- c(interesting,ont)
    }
    else {
      uninteresting <- c(uninteresting,ont)
    }
  }
  interestingdata <- subset(data, data$filename %in% interesting)
  uninterestingdata <- subset(data, data$filename %in% uninteresting)
  return(list("interesting"=interestingdata,"uninteresting"=uninterestingdata))
}
nofilter <- function(data) {
  return(data)
}

#Plot reasonerfingerprints
plotowlapiphases <- function(data_all,avg,filter) {
  data <- data_all$phasedata
  reasonerFingerprint <- data.frame(avg_loadtime=numeric(0),avg_reasonercreatetime=numeric(0),
                                    avg_classificationtime=numeric(0),avg_disposereasoner=numeric(0),
                                    reasoner=character(0))
  
  data_filtered <- filter(data)
  
  for(r in unique(data_filtered$reasoner)) {
    dataByReasoner <- subset(data_filtered, data_filtered$reasoner==r)
    #print(head(dataByReasoner$reasoner),3)
    results <- apply(dataByReasoner[owlapiphases], 2, avg)
    results$reasoner <- r
    reasonerFingerprint <- rbind(reasonerFingerprint, data.frame(results))
    #print(results)
  }
  #print(reasonerFingerprint)
  reshapedReasonerFingerprint <- melt(reasonerFingerprint,id.vars = "reasoner")
  p.plot <- ggplot(reshapedReasonerFingerprint, aes(x = reasoner, y = value,fill=variable)) +
    geom_bar(stat='identity') + coord_flip()
  p.name <- paste("owlapiphases",as.character(substitute(avg)),"_",as.character(substitute(filter)),sep="")
  return(list("plot_1"=p.plot, "function_name"=p.name))
}
plotclassificationphases_mod <- function(data_all,avg,filter) {
  data <- data_all$phasedata
  reasonerFingerprint <- data.frame(avg_pptime=numeric(0),avg_mmcltime=numeric(0),
                                    avg_poptime=numeric(0),reasoner=character(0))
  
  data_filtered <- filter(data)
  
  for(r in unique(data_filtered$reasoner)) {
    #print(r)
    dataByReasonerT <- subset(data_filtered, data_filtered$reasoner==r)
    dataByReasoner <- dataByReasonerT[complete.cases(dataByReasonerT[mod_reasoner_phases]), ]
    #print(head(dataByReasoner$reasoner),3)
    if(nrow(dataByReasoner)>0) {
    results <- apply(dataByReasoner[mod_reasoner_phases], 2, avg)
    results$reasoner <- r
    reasonerFingerprint <- rbind(reasonerFingerprint, data.frame(results))
    #print(results)
    }
  }
  reshapedReasonerFingerprint <- melt(reasonerFingerprint,id.vars = "reasoner")
  p.plot <- ggplot(reshapedReasonerFingerprint, aes(x = reasoner, y = value,fill=variable)) +
    geom_bar(stat='identity') + coord_flip()
  p.name <- paste("classificationphases_mod_",as.character(substitute(avg)),"_",as.character(substitute(filter)),sep="")
  return(list("plot_1"=p.plot, "function_name"=p.name))
}
plotclassificationphases <- function(data_all,avg,filter) {
  data <- data_all$phasedata
  reasonerFingerprint <- data.frame(avg_pptime=numeric(0),avg_cctime=numeric(0),avg_protime=numeric(0),
                                    avg_travtime=numeric(0),avg_poptime=numeric(0),reasoner=character(0))
  
  data_filtered <- filter(data)
  
  for(r in unique(data_filtered$reasoner)) {
    #print(r)
    dataByReasonerT <- subset(data_filtered, data_filtered$reasoner==r)
    dataByReasoner <- dataByReasonerT[complete.cases(dataByReasonerT[mon_reasoner_phases]), ]
    #print(head(dataByReasoner$reasoner),3)
    if(nrow(dataByReasoner)>0) {
      results <- apply(dataByReasoner[mon_reasoner_phases], 2, avg)
      results$reasoner <- r
      reasonerFingerprint <- rbind(reasonerFingerprint, data.frame(results))
      #print(results)
    }
  }
  #print(reasonerFingerprint)
  reshapedReasonerFingerprint <- melt(reasonerFingerprint,id.vars = "reasoner")
  p.plot <- ggplot(reshapedReasonerFingerprint, aes(x = reasoner, y = value,fill=variable)) +
    geom_bar(stat='identity') + coord_flip()
  p.name <- paste("classificationphases_",as.character(substitute(avg)),"_",as.character(substitute(filter)),sep="")
  return(list("plot_1"=p.plot, "function_name"=p.name))
}
plotmultiplot <- function(data,name) {
  plotlist <- list()
  i <- 1
  xlim1 <- ceiling(max(data[name],na.rm=TRUE))
  xlim0 <- floor(min(data[name],na.rm=TRUE))
  for(r in unique(data$reasoner)) {
    subset_a <- subset(data, data$reasoner==r)
    subset_d <- subset(subset_a,!is.na(subset_a[name]))
    if(nrow(subset_d)>0) {
      p1 <- ggplot(subset_d, aes_string(name)) + geom_histogram() + ggtitle(paste(r,sep="")) + xlim(xlim0,xlim1) + theme(axis.title.x = element_blank(),axis.title.y = element_blank())
      plotlist[[i]] <- p1
      i <- i+1
    }
  }
  row = ceiling(length(plotlist)/2)
  p.plot <- do.call(arrangeGrob, c(plotlist, list(nrow=row, ncol=2,main=name)))
  return(list("plot_1"=p.plot, "function_name"=name))
}
plotcorrelationmultiplot <- function(data,name,name2) {
  plotlist <- list()
  i <- 1
  xlim1 <- ceiling(max(data[name],na.rm=TRUE))
  xlim0 <- floor(min(data[name],na.rm=TRUE))
  if(xlim1==xlim0) {
    p.plot <- emptyplot
  }
  else {
  for(r in unique(data$reasoner)) {
    subset_a <- subset(data, data$reasoner==r)
    subset_d <- subset(subset_a,!is.na(subset_a[name]))
    if(nrow(subset_d)>0) {
      p1 <- ggplot(subset_d, aes_string(x=name, y=name2)) + geom_point(shape=1) + ggtitle(paste(r,sep=""))
      #p1 <- ggplot(subset_d, aes_string(name)) + geom_histogram() + ggtitle(paste(r,sep="")) + xlim(xlim0,xlim1) + theme(axis.title.x = element_blank(),axis.title.y = element_blank())
      plotlist[[i]] <- p1
      i <- i+1
    }
  }
  row = ceiling(length(plotlist)/2)
  p.plot <- do.call(arrangeGrob, c(plotlist, list(nrow=row, ncol=2,main=name)))
  }
  return(list("plot_1"=p.plot, "function_name"=paste("corrmulti",name,"_",name2,sep="")))
}
plotcltimeeffect <- function(data,column,name) {
  plotlist <- list()
  i <- 1
  for(r in unique(data$reasoner)) {
    subset_a <- subset(data, data$reasoner==r)
    subset_d <- as.data.frame(subset_a[complete.cases(subset_a[column]), column])
    names(subset_d) <- column
    if(nrow(subset_d)>0) {
      subset_d$cltimebinned <- apply(subset_d, 1,bineffect)
      p1 <- ggplot(subset_d,aes(x = factor(cltimebinned,levels=c("---","--","-","+","++","+++")))) + geom_bar(stat = "bin") + ggtitle(r) + scale_x_discrete(drop=FALSE) + xlab("Overall effect")
      plotlist[[i]] <- p1
      i <- i+1
    }
  }
  row = ceiling(length(plotlist)/2)
  p.plot <- do.call(arrangeGrob, c(plotlist, list(nrow=row, ncol=2,main=name)))
  return(list("plot_1"=p.plot, "function_name"=paste("cltime_",name,sep="")))
}
plotsattesthardness <- function(data,name) {
  plotlist <- list()
  i <- 1
  column <- c("sat_ct_avoided_mono","sat_ct_added_mono" ,"sat_ct_easier_overall","sat_ct_harder_overall","sat_ct_same_overall")
  labelsy <- c("Avoided","Added","Easier","Harder","Same")
  
  for(r in unique(data$reasoner)) {
    subset_a <- subset(data, data$reasoner==r)
    subset_d <- as.data.frame(subset_a[complete.cases(subset_a[column]), column])
    names(subset_d) <- column
    
    if(nrow(subset_d)>0) {
      results <- as.data.frame(colSums(subset_d))
      names(results) <- "value"
      results$cols <- labelsy
      p1 <- ggplot(results,aes(x=cols,y=value)) + geom_bar(stat = "identity") + ggtitle(r) + theme(axis.title.x = element_blank(),axis.title.y = element_blank())
      plotlist[[i]] <- p1
      i <- i+1
    }
  }
  row = ceiling(length(plotlist)/2)
  p.plot <- do.call(arrangeGrob, c(plotlist, list(nrow=row, ncol=2,main=name)))
  return(list("plot_1"=p.plot, "function_name"=paste("sat_hardness_",name,sep="","ctplot"=i)))
}

bineffect <- function(x) {
  if(x>(10)) {
    "+++"
  }
  else if(x>2) {
    "++"
  }
  else if(x>1) {
    "+"
  }
  else if(x<(-10)) {
    "---"
  }
  else if(x<(-2)) {
    "--"
  }
  else if(x<(-1)) {
    "-"
  }
  else {
    "0"
  }
}

extractphasedatafromdir<-function(files) {
  overall<- data.frame(pp=numeric(),cc=numeric(),prp=numeric(),st=numeric(),pop=numeric(),cltime=numeric(),reasoner=character(),ontology=character(),experimentid=character(),
                       stringsAsFactors=FALSE)
  subtest<- data.frame(testduration=numeric(),sub=character(),super=character(),reasoner=character(),ontology=character(),result=character(),experimentid=character(),
                       stringsAsFactors=FALSE)
  
  rel_cm<-c("pp","cc","prp","st","pop","cltime","reasoner","filename","experiment_runid")
  rel_sub<-c("testduration","sub","super","reasoner","filename","pos","experiment_runid")
  
  for(x in files) {
    print(x)
    cm_file<-paste(x,"/classificationmetadata.csv",sep="")
    s_files <-list.files(x,pattern="^(subsumptionmetadata)\\.*",full.names=TRUE)
    sub_data <- data
    
    sub_out <- do.call(rbind, lapply(s_files, function(x) {
      numeric=c("testduration")
      tt <- read.csv(x)
      tt[,numeric] <- sapply(tt[,numeric], as.character)
      tt[,numeric] <- sapply(tt[,numeric], as.numeric)
      tt
    }))
    
    print(cm_file)
    if(file.exists(cm_file)) {
      cm <- read.csv(cm_file) # load file  
      numeric<-c("PP_TS","CC_TS","PRP_TS","ST_TS","POP_TS","POPFIN_TS")
      cm[,numeric] <- sapply(cm[,numeric], as.character)
      cm[,numeric] <- sapply(cm[,numeric], as.numeric)
      cm$cltime<-cm$POPFIN_TS-cm$PP_TS
      cm$pp<-cm$CC_TS-cm$PP_TS
      cm$cc<-cm$PRP_TS-cm$CC_TS
      cm$prp<-cm$ST_TS-cm$PRP_TS
      cm$st<-cm$POP_TS-cm$ST_TS
      cm$pop<-cm$POPFIN_TS-cm$POP_TS
      overall<-rbind(overall,cm[rel_cm])
      if(!is.null(sub_out)) {
        print("SUB:")
        subtest<-rbind(subtest,sub_out[rel_sub])
      }
    }
      
    
  }
  return(list(overall=overall,subtest=subtest))
}


plotsattests<- function(pdata,changed,name) {
  atoms<-rownames(pdata)
  x<-1
  p.plot <- ggplot(pdata,aes(atom)) + ylab("Testtime (in ns)") + xlab("Atoms in path") + scale_x_discrete(labels=atoms)
  for (i in names(pdata)) {
    if(!(i=="atom")) {
      if(length(which(!is.na(pdata[[i]])))>1) { 
      colo<-"#000000"
      if(i %in% changed) {
        colo<-"#ff0000"
      }
      p.plot <- p.plot + geom_line(aes_string(y = i), colour = colo)
      x<-0
      }
    }
  } 
  if(x==1) {
   return(emptyplot())
  }
  else {
    return(list("plot_1"=p.plot, "function_name"=paste("sathardness_",name,sep = "")))   
  } 
}

featureScale<-function(x) {
  #http://en.wikipedia.org/wiki/Feature_scaling
  data<-x
  for(col in names(data)) {
    #print(col)
    if(!(col=="atom")) {
      dc<-data[[col]]
      data[col]<-normalizedZeroOne(dc)
    }
  }
  return(data)
}

rearrangeSathardResults<-function(data) {
  atoms<-colnames(data)
  atoms<-atoms[-1]
  
  xrange <- range(atoms) 
  yrange <- range(data[atoms])
  
  pdata<-as.data.frame(t(data[atoms]))
  cnames<-gsub("http://", "", data$id)
  cnames<-gsub("\\{|\\/|:|#|-", "", cnames)
  colnames(pdata)<-cnames
  
  pdata$atom<-seq(1:length(atoms))
  return(pdata)
}

fillMissingValues<-function(data) {
  affectedcolumns<-c()
  
  for(col in names(data)) {
    #print(col)
    if(!(col=="atom")) {
    dc<-data[[col]]
    ct<-length(dc)
    for(i in 1:ct) {
      this<-dc[i]
      if(i>1&i<ct) {
      
      if(is.na(this)) {
      prev<-lastnumeric(dc[1:i])
      nex<-firstnumeric(dc[i:ct])
      if(is.numeric(prev)&is.numeric(nex)) {
        mean<-(prev+nex)/2
        data[i,col]<-mean
        affectedcolumns<-c(affectedcolumns,col)
      }
      }
      }
    }
  }
  }
  return(list("changedcols"=unique(affectedcolumns),"changeddata"=data))
}

analysepath<-function(data,idvar,path,unit="ns") {
  x<-data[ , path]
  out<-data[c(idvar)]
  #out$slope<-apply(x, 1, getSlope)
  out$monotonic<-apply(x, 1, isMonotonic)
  out$dirchange_simple<-apply(x, 1, dirChangeSimple)
  out$dirchange_cat<-apply(x, 1, dirChangeCat,unit=unit)
  out$change_avg<-apply(x, 1, changeAvg)
  
  out$effect_mag<-apply(x, 1, effectMagnitude)
  out$effect_rel<-apply(x, 1, effectReliability)
  out$effect_tendency<-apply(x, 1, effectTendency)
  
  out$mean_path<-apply(x, 1, mean, na.rm = TRUE) 
  out$sd_path<-apply(x, 1, sd, na.rm = TRUE) 
  out$se_path<-apply(x, 1, std_error) 
  out$ct_nna_path<-apply(x, 1, ctNotNA)
  out$min_path<-apply(x, 1, min, na.rm = TRUE) 
  out$max_path<-apply(x, 1, max, na.rm = TRUE) 
  out$varcoef_path<-apply(x, 1, co.var)
  return(out)
}

ctNotNA<-function(x) {
  return(sum(!is.na(x)))
}

bestfit<-function(x) {
  if(sum( !is.na(x) )>1) {
    n<-1:length(x)
    line<-lm(x~n,na.action=na.exclude)
    pol2<-lm(x~I(n^2),na.action=na.exclude)
    pol3<-lm(x~I(n^3),na.action=na.exclude)
    linr2<-summary(line)$r.squared
    polr2<-summary(pol2)$r.squared
    polr3<-summary(pol2)$r.squared
    
    if(linr2>polr2) {
      return("lin")
    }
    else {
      return("quad")
    }
    
    return(line$coefficients[2])
  }
  else {
    return(NA)
  }
}

getSlope<-function(x) {
  n<-1:length(x)
  if(sum( !is.na(x) )>1) {
  line<-lm(x~n,na.action=na.exclude)
  return(line$coefficients[2])
  }
  else {
    return(NA)
  }
}

dirChangeSimple<-function(y) {
  ct_change<-0 
  direction<-"none"
  x<-y[!is.na(y)] #strip na
  x<-x[(!x==0)] #strip 0
  if(length(x)>1) {
  for(i in 1:(length(x)-1)) {
    this<-x[i]
    nexti<-x[i+1]
    nFoldChange<-foldChangeNormalised(this,nexti)
    if(nFoldChange>(1.05)) {
      if(direction=="down") {
        ct_change<-ct_change+1
      }
      direction<-"up"
    }
    else if(nFoldChange<(-1.05)) {
      if(direction=="up") {
      ct_change<-ct_change+1
      }
      direction<-"down"
    }
  }
  return(ct_change/(length(x)-1))
  }
  else {
    return(NA)
  }
}

dirChangeCat<-function(y,unit) {
  ct_change<-0
  x<-y[!is.na(y)] # strip na
  x<-x[(!x==0)] #strip 0
  if(length(x)>1) {
  for(i in 1:(length(x)-1)) {
    this<-x[i]
    nexti<-x[i+1]
    thiscat<-classifytesthardness(this,unit)
    nextcat<-classifytesthardness(nexti,unit)
    if(thiscat!=nextcat) {
    ct_change<-(ct_change+1)
    }
  }
  return(ct_change/(length(x)-1))
  }
  else{
    return(NA)
  }
}

changeAvg<-function(y) {
  change<-changeVector(y)
  return(mean(change,na.rm = TRUE))
}

changeSd<-function(y) {
  change<-changeVector(y)
  return(sd(change,na.rm = TRUE))
}

changeVector<-function(y) {
  change<-c()
  x<-y[!is.na(y)] #strip na
  x<-x[(!x==0)] #strip 0
  if(length(x)>1) {
  for(i in 1:(length(x)-1)) {
    this<-x[i]
    nexti<-x[i+1]
    nFoldChange<-foldChangeNormalised(this,nexti)
    change<-c(change,nFoldChange)
  }
  return(illegalNormalisation(change))
  }
  else {
    return(c())
  }
}

effectMagnitude<-function(x) {
  avgchange<-abs(changeAvg(x))
  changeclass<-"none"
  if(is.na(avgchange)) {
    changeclass<-"none"
  }
  else if(avgchange<0.1) {
    changeclass<-"low"
  }
  else if(avgchange<1) {
    changeclass<-"medium"
  }
  else {
    changeclass<-"high"
  }
  return(changeclass)
}

illegalNormalisation<-function(y) {
  # only use this function in the greates of needs. The idea is to rescale
  # values that represent a normalised (fold) change: These cannot take values -1 and 1. To
  # circumvent the obvious problems of standard deviation (its always at least 1) or mean (meaningless means between -1 and 1)
  out<-c()
  x<-y
  for(a in x){
    if(is.na(a)) {
      out<-c(out,NA)
    }
    else if(a>0){
      out<-c(out,(a-1))
    }
    else {
      out<-c(out,(a+1))
    }
  }
  return(out)
}

growthchange<-function(x,y){
  return(illegalNormalisation(foldChangeNormalised(x,y)))
}

effectReliability<-function(x) {
  mon<-isMonotonic(x)
  sd_change<-sd(changeVector(x))
  dcs<-dirChangeSimple(x)
    
  changeclass<-"none"
  if(is.na(mon)) {
    changeclass<-"none"
  }
  else if(mon) {
    changeclass<-"very high"
  }
  else if(dcs==0) {
    changeclass<-"high"
  }
  else if(sd_change<0.5) {
    changeclass<-"medium"
  }
  else {
    changeclass<-"low"
  }
  return(changeclass)
}

effectTendency<-function(x) {
  change<-changeAvg(x)
  tendency<-"none"
  if(is.na(change)) {
    tendency<-"none"
  }
  else if(change>0.05) {
    tendency<-"positive"
  }
  else if(change<0.05) {
    tendency<-"negative"
  }
  else {
    tendency<-"neutral"
  }
  return(tendency)
}

isMonotonic<-function(x) {
  
  rising<-NA
  y<-x[!is.na(x)]
  if(length(y)<2) {
    return(NA)
  }
  if(y[1]>y[2]) {
    rising<-FALSE
  } else {
    rising<-TRUE
  }
  prev<-0
  for(i in y) {
    if(prev==0) {
      prev<-i
    }
    else {
    if(i<prev) {
      if(rising) {
        return(FALSE)
      }
      prev<-i
    }
    else if(i>prev) {
      if(!rising) {
        return(FALSE)
      }
      prev<-i
    }
    else {
      #if it happens to be the same, dont do anything
      prev<-i
    }
    }
  }
  return(TRUE)
}

lastnumeric<-function(x){
  y<-rev(x)
  for(i in y) {
    if(!is.na(i)) {
      if(is.numeric(i)) {
      return(i)
      }
    } 
  }
  return(NA)
}

firstnumeric<-function(x){
  for(i in x) {
    if(!is.na(i)) {
      if(is.numeric(i)) {
        return(i)
      }
    } 
  }
  return(NA)
}

classifytesthardness<-function(x,unit="ns") {
  unitre<-ifelse(unit=="sec",1000000000,ifelse(unit=="ms",1000000,ifelse(unit=="hrs",3600000000000,ifelse(unit=="min",60000000000,1))))
  x<-x*unitre
  if(is.na(x)) {
    return(NA)
  }
  else if(x>100000000000) {
    return(8)
  }
  else if(x>10000000000) {
    return(7)
  }
  else if(x>1000000000) {
    return(6)
  }
  else if(x>100000000) {
    return(5)
  }
  else if(x>10000000) {
    return(4)
  }
  else if(x>1000000) {
    return(3)
  }
  else if(x>100000) {
    return(2)
  }
  else {
    return(1)
  }
}

getallsubsumptiontestdata<-function(files){
  # A record corresponds to a RUN of a subsumption test by a reasoner in a module of an ontology.
  outmod<-data.frame(pp=numeric(),cc=numeric(),prp=numeric(),st=numeric(),pop=numeric(),cltime=numeric(),reasoner=character(),module=character(),experimentid=character(),ontology=character(),stringsAsFactors=FALSE) 
  outst<-data.frame(id=character(),sub=character(),super=character(),ontology=character(),reasoner=character(),module=character(),testduration=numeric(),testresult=logical(),testhardness_cat=numeric(),experimentid=character(),stringsAsFactors=FALSE) 
  for(file in files) {
    filesont <- list.dirs(file,all.dirs=TRUE,full.names=TRUE,pattern="^[a-zA-Z0-9]\\.*")
    res<-extractphasedatafromdir(filesont)
    overall<-res$overall
    subtest<-res$subtest
    fname<-sub(".*/", "", file)
    overall$ontname<-fname
    sub_complete <- subtest[0,]
    if(nrow(subtest)>0) {
      subtest$ontname<-fname
      subtest$id<-get_subtest_id(subtest)
      sub_complete <- subtest[complete.cases(subtest),]
      sub_complete$hardcat<-sapply(sub_complete$testduration,classifytesthardness)
      outst <- rbind(outst, sub_complete[c("id","sub","super","ontname","reasoner","filename","testduration","pos","hardcat","experiment_runid")])
    }
    overall_complete <- overall[complete.cases(overall),]
    outmod <- rbind(outmod, overall_complete[c("pp","cc","prp","st","pop","cltime","reasoner","filename","experiment_runid","ontname")])
    
  } #end for through files
  return(list("stdata"=outst,"moddata"=outmod))
}


descStats<-function(vector){  
  out<-data.frame(mean=numeric(),median=numeric(),sd=numeric(),sde=numeric(),min=numeric(),max=numeric(),ctmeasurements=numeric(),ctna=numeric(),coevar=numeric())
  if(is.null(vector)|length(vector)==0) {
    out[1,]<-c(NA,NA,NA,NA,NA,NA,NA,NA,NA)  
  }
  else {
    out[1,]<-c(mean(vector,na.rm = TRUE),median(vector,na.rm = TRUE),sd(vector,na.rm = TRUE),std_error(vector),min(vector,na.rm = TRUE),max(vector,na.rm = TRUE),length(vector),length(vector[is.na(vector)]),co.var(vector))
  }
  return(out)
}

get_subtest_id<-function(subtest){
  return(paste(subtest$sub,subtest$super,sep="{"))
}

addModuleStats<-function(rec_out,vec,variablename,id,mergeby) {
  # CT: Total Classificationtime 
  vec_agg<-descStats(vec)
  names(vec_agg) <- paste(variablename,"_",names(vec_agg),sep="")
  vec_agg[mergeby]<-id
  rec_out <- merge(rec_out,vec_agg,by=c(mergeby))
  return(rec_out)
}

intramoduleanalysis<-function(subtestdata,moduledata){
  # A record corresponds to a subsumption test by a reasoner in a module of an ontology.
  # Aggregates across ROM-run (Reasoner,Ontology,Module)
  moduleout<-data.frame()
  moduleout_index<-1
  stout<-data.frame()
  stout_index<-1
  
  total_comp<-length(unique(subtestdata$filename))
  comp_count<-0
  
  for(o in unique(subtestdata$ontname)) {
    sub_o<-subset(moduledata,moduledata$ontname==o)
    sub_ost<-subset(subtestdata,subtestdata$ontname==o)
    for(r in unique(subtestdata$reasoner)) {
      sub_or<-subset(sub_o,sub_o$reasoner==r)
      sub_rost<-subset(sub_ost,sub_ost$reasoner==r)

      for(m in unique(sub_or$filename)) {
        comp_count<-comp_count+1
        print(paste("Reasoner:",r,"Ontology:",o,"Module",m,comp_count,"out of",total_comp))
        
        sub_rom<-subset(sub_or,sub_or$filename==m)
        sub_romst<-subset(sub_rost,sub_rost$filename==m)
        # Module aggregation
        rec_out<-data.frame(module=character(),ontology=character(),reasoner=character(),path=integer(),stringsAsFactors = FALSE)
        p<-unique(sub_rom$path)
        rec_out[1,]<-c(m,o,r,p)
        
        # CT: Total Classificationtime 
        rec_out<-addModuleStats(rec_out,sub_rom$OVERALLCLTIME,"cltime",m,"module")
      
        sum_st<-c()
        sum_st_pos<-c()
        sum_st_neg<-c()
        count_st<-c()
        count_st_pos<-c()
        count_st_neg<-c()
        sst_cltime<-c()
        sst_cltime_pos<-c()
        sst_cltime_neg<-c()
      
        if(nrow(sub_romst)>0) {
        for(run in unique(sub_romst$experiment_runid)) {
          rundata_mod<-subset(sub_rom,sub_rom$experiment_runid==run)
          rundata_st<-subset(sub_romst,sub_romst$experiment_runid==run)
          if(nrow(rundata_mod)!=1) {
            stop("intramoduleanalysis: Illegal number of rows for rundata_mod")
          }
          
          cltime_run<-rundata_mod$OVERALLCLTIME
          
          # SST: Sum(Subtests) 
          sum_st_i<-sum(rundata_st$testduration,na.rm = TRUE)
          sum_st<-c(sum_st,sum_st_i)
          if(is.null(sum_st_i)) {
            stop("isNULL???")
          }
          
          # SST-POS: Sum(Subtests) 
          sum_st_pos_i<-sum(rundata_st[rundata_st$pos==1,]$testduration,na.rm = TRUE)
          sum_st_pos<-c(sum_st_pos,sum_st_pos_i)
          
          # SST-NEG: Sum(Subtests) 
          sum_st_neg_i<-sum(rundata_st[rundata_st$pos==0,]$testduration,na.rm = TRUE)
          sum_st_neg<-c(sum_st_neg,sum_st_neg_i)
          
          # SST/CT: Ratio of testtime to classificationtime
          sst_cltime<-c(sst_cltime,sum_st_i/cltime_run)
          
          # SSTPOS/CT: Ratio of positive tests to classificationtime
          sst_cltime_pos<-c(sst_cltime_pos,sum_st_pos_i/cltime_run)
          
          # SSTNEG/CT: Ratio of negative tests to classificationtime
          sst_cltime_neg<-c(sst_cltime_neg,sum_st_neg_i/cltime_run)
          
          # SST: Sum(Subtests) 
          count_st_i<-sum(!is.na(rundata_st$pos),na.rm = TRUE)
          count_st<-c(count_st,count_st_i)
          if(is.null(count_st_i)) {
            stop("isNULL???")
          }
          
          # SST-POS: Sum(Subtests) 
          count_st_pos_i<-sum(rundata_st$pos==1,na.rm = TRUE)
          count_st_pos<-c(count_st_pos,count_st_pos_i)
          
          # SST-NEG: Sum(Subtests) 
          count_st_neg_i<-sum(rundata_st$pos==0,na.rm = TRUE)
          count_st_neg<-c(count_st_neg,count_st_neg_i)
        }
        }
      
      rec_out<-addModuleStats(rec_out,sum_st,"sum_st",m,"module")
      rec_out<-addModuleStats(rec_out,sum_st_pos,"sum_st_pos",m,"module")
      rec_out<-addModuleStats(rec_out,sum_st_neg,"sum_st_neg",m,"module")
      
      rec_out<-addModuleStats(rec_out,count_st,"count_st",m,"module")
      rec_out<-addModuleStats(rec_out,count_st_pos,"count_st_pos",m,"module")
      rec_out<-addModuleStats(rec_out,count_st_neg,"count_st_neg",m,"module")
      
      rec_out<-addModuleStats(rec_out,sst_cltime,"sst_cltime",m,"module")
      rec_out<-addModuleStats(rec_out,sst_cltime_pos,"sst_cltime_pos",m,"module")
      rec_out<-addModuleStats(rec_out,sst_cltime_neg,"sst_cltime_neg",m,"module")      
      
      moduleout<-rbindif(moduleout,rec_out,moduleout_index)
      moduleout_index<-moduleout_index+1
      print(paste(m,r,o))
        # ST aggregation 
        
        for(st in unique(sub_romst$id)) {
            rec_out_st<-data.frame(module=character(),ontology=character(),reasoner=character(),id=character(),stringsAsFactors = FALSE)
            rec_out_st[1,]<-c(m,o,r,st)
            
            sub_atom_st<-subset(sub_romst,sub_romst$id==st)  
            st_all<-sub_atom_st$testduration
            st_hardcat<-sub_atom_st$hardcat
            
            rec_out_st<-addModuleStats(rec_out_st,st_all,"testduration",st,"id")
            rec_out_st$max_hardcat<-max(st_hardcat,na.rm = TRUE)
            rec_out_st$min_hardcat<-min(st_hardcat,na.rm = TRUE)
            rec_out_st$ct_distinct_hardcat<-length(unique(st_hardcat))
            
            shapiro<-NA
            if(length(st_all[!is.na(st_all)])>2&length(unique(st_all[!is.na(st_all)]))>1) {
              shapiro<-shapiro.test(sub_atom_st$testduration)$p.value
            }
            rec_out_st$shapiro<-shapiro
            stout<-rbindif(stout,rec_out_st,stout_index)
            stout_index<-stout_index+1
          } # end of st
    } # end for through modules
  } #end for through reasoner
  #end for through ontologies
  }
  return(list("modules"=moduleout,"subsumptiontests"=stout))
}

getpairsofmodulesbyindex<-function(ontdata) {
  moduleout<-data.frame(compid=character(),ontology=character(),sub=character(),super=character(),path=character(),stringsAsFactors = FALSE) 
  imodout<-1
  
  if(!("ontname" %in% names(ontdata))) {
    ontdata$ontname<-ontdata$ontology
  }
  
  for(o in unique(ontdata$ontname)) {
    sub_o<-subset(ontdata,ontdata$ontname==o)
    
    for(r in unique(sub_o$reasoner)) {
      sub_or<-subset(sub_o,sub_o$reasoner==r)
      sub_oro<-subset(sub_or,is.na(sub_or$path))
      
      for(p in unique(sub_or$path)) {
        sub_por<-subset(sub_or,sub_or$path==p)
      for(m in unique(sub_por$filename)) {
        sub_rom<-subset(sub_por,sub_or$filename==m)
        index1<-unique(sub_rom$mod_index)
        for(m2 in unique(sub_por$filename)) {
          sub_rom2<-subset(sub_por,sub_por$filename==m2)
          index2<-unique(sub_rom2$mod_index)
          if(index1>index2) {
            id<-paste(m2,"{",m,sep="")
            if(!(id %in% moduleout$compid)) {
              moduleout[imodout,]<-c(id,o,m2,m,p)
              imodout<-imodout+1
            }
          }
        }
        if(nrow(sub_oro)>0) {
        om<-sub_oro$filename[1]
        id<-paste(m,"{",om,sep="")
        if(!(id %in% moduleout$compid)) {
          moduleout[imodout,]<-c(id,o,m,om,p)
          imodout<-imodout+1
        }
        }
      }
      }
    }
  }
  return(moduleout)
}

getpairsofmodules<-function(paths) {
  moduleout<-data.frame(compid=character(),ontology=character(),sub=character(),super=character(),path=character(),stringsAsFactors = FALSE) 
  imodout<-1
  
  #order predeccesors/successors
  for(pathname in paths) {
    o<-sub("/.*","",sub(".*/paths/*", "", pathname))
    p<-read.csv(pathname,header = FALSE,stringsAsFactors=F)
    path<-p$V1
    pname<-sub(".*/", "", pathname)
    for(i in 1:length(path)) {
      if(length(path)>i) {
        pre<-path[i]
        succ<-path[i+1]
        id<-paste(o,"|",pre,"|",succ,sep="")
        if(!(id %in% moduleout$compid)) {
          moduleout[imodout,]<-c(id,o,pre,succ,pname)
          imodout<-imodout+1
        }
      }
    }
  } #order predeccesors/successors done
  return(moduleout)
}

intermoduleanalysis<-function(subtestdata,moduledata,modulepairs) {
  moduleout<-data.frame(stringsAsFactors = FALSE)
  stout<-data.frame(stringsAsFactors = FALSE) 
  moduleout_index<-1
  stout_index<-1
    
  for(r in unique(subtestdata$reasoner)) {
    mod_r<-subset(moduledata,moduledata$reasoner==r)
    sub_r<-subset(subtestdata,subtestdata$reasoner==r)
    for(o in unique(sub_r$ontology)) {
    mod_ro<-subset(mod_r,mod_r$ontology==o)
    sub_ro<-subset(sub_r,sub_r$ontology==o)    
    
    rel_modulepairs<-subset(modulepairs,modulepairs$ontology==o)
    total_comp<-nrow(rel_modulepairs)
    comp_count<-0
    
      for(id in rel_modulepairs$compid) {
        comp_count<-comp_count+1
        print(paste("Reasoner:",r,"Ontology:",o,"Comparison",comp_count,"out of",total_comp))
        mod_comp_id<-subset(rel_modulepairs,rel_modulepairs$compid==id)
        st_comp_id<-mod_comp_id
        
        if(nrow(mod_comp_id)!=1) {
          stop("Illegal number of records in intermoduleanalysis (mod_comp_id)")
        }
        sub<-mod_comp_id$sub
        super<-mod_comp_id$super
        data_sub<-subset(mod_ro,mod_ro$module==sub)
        data_super<-subset(mod_ro,mod_ro$module==super)
        if(nrow(data_sub)>1|nrow(data_super)>1) {
          stop("Illegal number of records in intermoduleanalysis (data_sub,data_super)")
        } else if(nrow(data_sub)==0|nrow(data_super)==0) {
          print("No entries, skip.. (data_sub,data_super)")
          next
        }
        #COMPARISON-MODULES: CLTIME (Difference and FoldChange)
        print("COMPMOD:CLTIME")
        mod_comp_id<-getcomparison(data_sub,data_super,"cltime",mod_comp_id)
        
        #COMPARISON-MODULES: SUMSUBTESTTIME (Difference and FoldChange)
        print("COMPMOD:SUMSAT")
        mod_comp_id<-getcomparison(data_sub,data_super,"sum_st",mod_comp_id)
        
        #COMPARISON-MODULES: SUMSUBTESTCOUNTS (Difference and FoldChange)
        print("COMPMOD:SUBTESTCOUNTS")
        mod_comp_id<-getcomparison(data_sub,data_super,"count_st",mod_comp_id)
        mod_comp_id<-getcomparison(data_sub,data_super,"count_st_pos",mod_comp_id)
        mod_comp_id<-getcomparison(data_sub,data_super,"count_st_neg",mod_comp_id)
        mod_comp_id$reasoner<-r
        moduleout<-rbindif(moduleout,mod_comp_id,moduleout_index)
        moduleout_index<-moduleout_index+1
        
        # COMPARISON Subsumptiontests
        
        data_st_sub<-subset(sub_ro,sub_ro$module==sub)
        data_st_super<-subset(sub_ro,sub_ro$module==super)
        tests<-c(unique(data_st_sub$id),unique(data_st_super$id))
        st_sub_present<-FALSE
        st_super_present<-FALSE
        
        print("STTESTCOMP")
        test_ct<-0
        totaltests<-length(tests)
        for(st in tests) {
          test_ct<-test_ct+1
          #print(paste("Test",test_ct,"of",totaltests))
          sd_comp_out<-st_comp_id
          sd_comp_out$subtestid<-st
          data_st_sub_st<-subset(data_st_sub,data_st_sub$id==st)
          data_st_super_st<-subset(data_st_super,data_st_super$id==st)
                    
          if(nrow(data_st_sub_st)>1) {
            stop("intramoduleanalysis, wrong number of records for data_st_sub_st")
          }
          else if(nrow(data_st_sub_st)==0) {
            sd_comp_out$st_sub_present<-st_sub_present<-FALSE
          }
          else {
            sd_comp_out$st_sub_present<-st_sub_present<-TRUE
            sd_comp_out$sub<-data_st_sub_st$sub
            sd_comp_out$sub<-data_st_sub_st$super
          }
          
          if(nrow(data_st_super_st)>1) {
            stop("intramoduleanalysis, wrong number of records for data_st_super_st")
          }
          else if(nrow(data_st_super_st)==0) {
            sd_comp_out$st_super_present<-st_super_present<-FALSE
          }
          else {
            sd_comp_out$st_super_present<-st_super_present<-TRUE
            sd_comp_out$sub<-data_st_super_st$sub
            sd_comp_out$sub<-data_st_super_st$super
          }
          
          #COMPARISON-SUBSUMPTIONTESTTIME: SUBTESTDURATION (Difference and FoldChange)
          sub_st_testduration_min<-NA
          sub_st_testduration_max<-NA
          sub_st_testduration_mean<-NA
          super_st_testduration_min<-NA
          super_st_testduration_max<-NA
          super_st_testduration_mean<-NA
          
          if(st_sub_present) {
          sub_st_testduration_min<-data_st_sub_st$testduration_min
          sub_st_testduration_max<-data_st_sub_st$testduration_max
          sub_st_testduration_mean<-data_st_sub_st$testduration_mean
          }
          if(st_super_present) {
          super_st_testduration_min<-data_st_super_st$testduration_min
          super_st_testduration_max<-data_st_super_st$testduration_max
          super_st_testduration_mean<-data_st_super_st$testduration_mean
          }
          
          sd_comp_out$wc_diff<-sub_st_testduration_min-super_st_testduration_max
          sd_comp_out$bc_diff<-sub_st_testduration_max-super_st_testduration_min
          sd_comp_out$mean_diff<-sub_st_testduration_mean-super_st_testduration_mean
          
          sd_comp_out$wc_fchange<-growthchange(sub_st_testduration_min,super_st_testduration_max)
          sd_comp_out$bc_fchange<-growthchange(sub_st_testduration_max,super_st_testduration_min)
          sd_comp_out$mean_fchange<-growthchange(sub_st_testduration_mean,super_st_testduration_mean)
          
         #classification of stability
         if(st_sub_present&st_super_present){
         cl_stab<-classify_stability(data_st_sub_st,data_st_super_st,sd_comp_out$mean_fchange)
         sd_comp_out$tendency<-cl_stab$tendency
         sd_comp_out$magnitude<-cl_stab$magnitude
         sd_comp_out$stability<-cl_stab$stability
         sd_comp_out$sub_st_ct<-data_st_sub_st$testduration_ctmeasurements
         sd_comp_out$super_st_ct<-data_st_super_st$testduration_ctmeasurements
         sd_comp_out$counterintuitive<-is_counterintuitive(sd_comp_out)
         }
         else {
           sd_comp_out$tendency<-NA
           sd_comp_out$magnitude<-NA
           sd_comp_out$stability<-NA
           sd_comp_out$sub_st_ct<-NA
           sd_comp_out$super_st_ct<-NA
           sd_comp_out$counterintuitive<-NA
         }
         
         stout<-rbindif(stout,sd_comp_out,stout_index)
         stout_index<-stout_index+1
        } #end tests
      }  
    } #end ontologies
  }#end reasoner
  return(list("modules"=moduleout,"subsumptiontests"=stout))
}

is_counterintuitive<-function(sd_comp_out){
  ten<-sd_comp_out$magnitude
  
  if(ten=="Easier"){
    mag<-sd_comp_out$magnitude
    if(mag=="High"){
    stab<-sd_comp_out$magnitude
      if(stab!="Low") {
        ct_sub<-sd_comp_out$sub_st_ct
        if(ct_sub>1) {
        ct_super<-sd_comp_out$super_st_ct
        if(ct_super>1) {
          return(TRUE)
        }
      }
    }
  }
}
return(FALSE)
}

classify_stability<-function(data_st_sub_st,data_st_super_st,change){
 out<-data.frame(tendency=character(),magnitude=character(),stability=character(),stringsAsFactors = FALSE)
 
 tendency<-get_tendency(change)
 magnitude<-get_magnitude(change)
 
 if(tendency!="Neutral") {
 stability<-get_stability(data_st_sub_st$testduration_min,data_st_super_st$testduration_min,data_st_sub_st$testduration_max,data_st_super_st$testduration_max)
 }  else {
   if(data_st_sub_st$testduration_ctmeasurements<2) {
     stability<-"Low"
   } else if(data_st_super_st$testduration_ctmeasurements<2) {
       stability<-"Low"
  } else if(data_st_sub_st$testduration_coevar<(0.05)) {
     if(data_st_super_st$testduration_coevar<(0.05)) {
       stability<-"High"
     } else {
       stability<-"Low"
     }
     } else {
     stability<-"Low"
   }
 }
 out[1,]<-c(tendency,magnitude,stability)
 return(out)
}

get_stability<-function(min_v1,min_v2,max_v1,max_v2){
  v<-numeric()
  
  if(max_v1<=min_v2) {
    return("Clearcut")
  } else if(min_v1>=max_v2) {
    return("Clearcut")
  } 
  else if(max_v1>max_v2) {
    if(min_v1<min_v2) {
      return("Low")
    }
    else if(min_v1>min_v2) {
      v<-max_v2-min_v1
    }
    else {
      return("Low")
    }
  }
  else if(max_v1<max_v2) {
    if(min_v1>min_v2) {
      return("Low")
    }
    else if(min_v1<min_v2) {
      v<-max_v1-min_v2
    }
    else {
      return("Low")
    }
  }
  else {
    return("Low")
  }
  all<-c(min_v1,min_v2,max_v1,max_v2)
  range<-max(all)-min(all)
  overlap<-v/range
  if(overlap<(0.20)) {
  return("High")
  }
  else {
    return("Low")
  }
}

get_tendency<-function(x){
  if(x>(0.05)) {
    return("Harder")
  } else if(x<(-0.05)) {
    return("Easier")
  } else {
    return("Neutral")
  }
}

get_magnitude<-function(y){
  x<-abs(y)
  if(x>(0.50)) {
    return("High")
  } else if(x>(0.05)) {
    return("Medium")
  } else {
    return("Low")
  }
}


getcomparison<-function(data_sub,data_super,name,mod_comp_id){
  assign(paste("sub_",name,"_min",sep=""),data_sub[,paste(name,"_min",sep="")])
  assign(paste("sub_",name,"_max",sep=""),data_sub[,paste(name,"_max",sep="")])
  assign(paste("sub_",name,"_mean",sep=""),data_sub[,paste(name,"_mean",sep="")])
  assign(paste("super_",name,"_min",sep=""),data_super[,paste(name,"_min",sep="")])
  assign(paste("super_",name,"_max",sep=""),data_super[,paste(name,"_max",sep="")])
  assign(paste("super_",name,"_mean",sep=""),data_super[,paste(name,"_mean",sep="")])
  
   
  mod_comp_id[,paste("wc_diff_",name,sep="")]<-get(paste("sub_",name,"_min",sep=""))-get(paste("super_",name,"_max",sep=""))
  mod_comp_id[,paste("bc_diff_",name,sep="")]<-get(paste("sub_",name,"_max",sep=""))-get(paste("super_",name,"_min",sep=""))
  mod_comp_id[,paste("mean_diff_",name,sep="")]<-get(paste("sub_",name,"_mean",sep=""))-get(paste("super_",name,"_mean",sep=""))
  
  mod_comp_id[,paste("wc_fchange_",name,sep="")]<-growthchange(get(paste("sub_",name,"_min",sep="")),get(paste("super_",name,"_max",sep="")))
  mod_comp_id[,paste("bc_fchange_",name,sep="")]<-growthchange(get(paste("sub_",name,"_max",sep="")),get(paste("super_",name,"_min",sep="")))
  mod_comp_id[,paste("mean_fchange_",name,sep="")]<-growthchange(get(paste("sub_",name,"_mean",sep="")),get(paste("super_",name,"_mean",sep="")))
  return(mod_comp_id)
}

getAllPaths<-function(files,pathdir) {
  paths<-vector()
  for(file in files) {
    fname<-sub(".*/", "", file)
    pathsont<-list.files(paste(pathdir,fname,sep=""),full.names=TRUE,pattern="^[a-zA-Z0-9]\\.*")
    paths<-c(paths,pathsont)
  }
  return(paths)
}

rbindif<-function(df1,df2,rowcount) {
  if(ncol(df1)==0) {
    df<-data.table(df2)
  }
  else {
    df<-rbindlist(list(df1, df2))
  }
  return(df)
}

prepare_stdata<-function(stdatafile,inverthermit){
  #LOADING#
  stdata<-read.csv(stdatafile,colClasses = "character")
  if (inverthermit==TRUE) {
    print("INVERTING STDATA HERMIT RESULTS!")
    stdata$pos<-ifelse(stdata$reasoner=="HermiT[EXP]",ifelse(stdata$pos=="1","0","1"),stdata$pos)
  } 
  
  #TYPING#
  st_numeric<-c("testduration","starttime","endtime")
  stdata[, st_numeric] <- sapply(stdata[, st_numeric], as.numeric)
  #stdata[, c("pos")] <- sapply(stdata[, c("pos")], as.logical)
  
  #VARIABLES#
  stdata$hardcat<-sapply(stdata$testduration,classifytesthardness)
  stdata$hardcatname<-sapply(stdata$hardcat,binlabel)
  stdata[, c("hardcatname")] <- sapply(stdata[, c("hardcatname")], as.factor)
  stdata$ontname<-getontnamefrommodulename(stdata$filename)
  
  return(stdata)
}

prepare_ontdata_prepro<-function(preprometa){
  ontmetadata_pre<-read.csv(preprometa,head=TRUE,sep=",")
  return(ontmetadata_pre)
}

getontnamefrommodulename<-function(mnames){
  onames<-c()
  for(m in mnames){
    #m_bottom_vso.vital-sign-ontology.1.orig.owl.xml_indexed.owl.xml
    #m_bottom_path_0_cumulativesubset_10_16_3e45484d-e869-4aa0-9264-e9ec8642ad55_ntdo.neglected-tropical-disease-ontology.1.orig.owl.xml_indexed.owl.xml
    r<-gsub("^m_bottom_","",m)
    if(grepl("^(path)",r)){
      r<-gsub("^(.*?)_","",substr(r, start=58, stop=nchar(r)))
    }
    onames<-c(onames,r)
  }
  return(onames)
}

getontnamefromsubsetname<-function(mnames,short=FALSE){
  onames<-c()
  for(m in mnames){
    #m_bottom_vso.vital-sign-ontology.1.orig.owl.xml_indexed.owl.xml
    #m_bottom_path_0_cumulativesubset_10_16_3e45484d-e869-4aa0-9264-e9ec8642ad55_ntdo.neglected-tropical-disease-ontology.1.orig.owl.xml_indexed.owl.xml
    r<-paste(strsplit(m,"_")[[1]][5],collapse = "_")
    if(short){
      r<-get_ontology_short_name(r)
    }
    onames<-c(onames,r)
  }
  return(onames)
}

getontnamefromlongname<-function(mnames,num,short=FALSE){
  onames<-c()
  for(m in mnames){
    #m_bottom_vso.vital-sign-ontology.1.orig.owl.xml_indexed.owl.xml
    #m_bottom_path_0_cumulativesubset_10_16_3e45484d-e869-4aa0-9264-e9ec8642ad55_ntdo.neglected-tropical-disease-ontology.1.orig.owl.xml_indexed.owl.xml
    r<-paste(strsplit(m,"_")[[1]][num],collapse = "_")
    if(short){
      r<-get_ontology_short_name(r)
    }
    onames<-c(onames,r)
  }
  return(onames)
}


getmoduleindexfrommodulename<-function(mnames){
  onames<-c()
  for(m in mnames){
    #m_bottom_vso.vital-sign-ontology.1.orig.owl.xml_indexed.owl.xml
    #m_bottom_path_0_cumulativesubset_10_16_3e45484d-e869-4aa0-9264-e9ec8642ad55_ntdo.neglected-tropical-disease-ontology.1.orig.owl.xml_indexed.owl.xml
    r<-gsub("^m_bottom_","",m)
    index<-NA
    if(grepl("^(path)",r)){
      r<-gsub("_(.*)$","",gsub("^(.*?)_","",substr(r, start=16, stop=nchar(r))))
      x<-substr(gsub("^(path_)","",r),0,2)
      r<-x
      if(!is.na(as.numeric(r))) {
        index<-as.integer(r)
      }
      else {
        index<-as.integer(substr(r,0,1))
      }
    }
    onames<-c(onames,index)
  }
  return(onames)
}

getpathfrommodulename<-function(mnames){
  onames<-c()
  for(m in mnames){
    #m_bottom_vso.vital-sign-ontology.1.orig.owl.xml_indexed.owl.xml
    #m_bottom_path_0_cumulativesubset_10_16_3e45484d-e869-4aa0-9264-e9ec8642ad55_ntdo.neglected-tropical-disease-ontology.1.orig.owl.xml_indexed.owl.xml
    r<-gsub("^m_bottom_","",m)
    path<-NA
    if(grepl("^(path)",r)){
      x<-substr(gsub("^(path_)","",r),0,2)
      r<-x
      if(!is.na(as.numeric(r))) {
        path<-as.integer(r)
      }
      else {
        path<-as.integer(substr(r,0,1))
      }
      
    }
    onames<-c(onames,path)
  }
  return(onames)
}

prepare_ontdata<-function(ontdata,ontmetadata,stdata,hardesttest=FALSE,filesuffix=""){
  #TYPING#
  sat_ct_total_numeric<-c("sat_ct_total_01_veryhard","sat_ct_total_02_hard","sat_ct_total_03_mediumhard","sat_ct_total_04_medium","sat_ct_total_05_mediumeasy","sat_ct_total_06_easy","sat_ct_total_07_veryeasy","sat_ct_total_08_trivial")
  other_ontdata_numeric<-c("B2createreasonertime","B3classificationtime","tt_sat_all","sat_ct_total")
  ontdata[, sat_ct_total_numeric] <- sapply(ontdata[, sat_ct_total_numeric], as.numeric)
  ontdata[, other_ontdata_numeric] <- sapply(ontdata[, other_ontdata_numeric], as.numeric)
  
  ontmetadata[, c("tbox_size_incl","class_count_incl")] <- sapply(ontmetadata[, c("tbox_size_incl","class_count_incl")], as.numeric)
    
  #new variables
  ontdata$ontname<-getontnamefrommodulename(ontdata$filename)
  ontdata$mod_index<-getmoduleindexfrommodulename(ontdata$filename)
  ontdata$path<-getpathfrommodulename(ontdata$filename)
  ontdata$OVERALLCLTIME<-ontdata$B2createreasonertime+ontdata$B3classificationtime
  ontmetadata$filename<-paste(ontmetadata$filename,filesuffix,sep="")
  
  ontdata$wc_st_bin<-apply(ontdata[,sat_ct_total_numeric], 1, function(x) { binbytesttype(x) } )
  ontdata$rat_o<-100
  ontdata$rat_sum_sat<-(ontdata$tt_sat_all/ontdata$OVERALLCLTIME)*100
  ontdata$impact<-ifelse(ontdata$rat_sum_sat>=40,"strong",ifelse(ontdata$rat_sum_sat>=20,"medium",ifelse(ontdata$rat_sum_sat>0,"small","none")))
  
  if(hardesttest==TRUE) {
  ontdata$HARDESTTEST<-gethardesttest(stdata,ontdata$experiment_runid)
  ontdata$rat_max_sat<-(ontdata$HARDESTTEST/ontdata$OVERALLCLTIME)*100
  }
  
  #merge
  #ontdata <- merge(ontdata,ontmetadata,by="filename")
  #ontdata$n2<-ontdata$class_count_incl^2
  #ontdata$rat_n2_actual<-ontdata$sat_ct_total/ontdata$n2
  return(ontdata)
}

createplotbybin<-function(ontdata){
  p2<-createhistforwcstbybin(ontdata,1)
  p3<-createhistforwcstbybin(ontdata,2)
  p4<-createhistforwcstbybin(ontdata,3)
  p5<-createhistforwcstbybin(ontdata,4)
  p6<-createhistforwcstbybin(ontdata,5)
  p7<-createhistforwcstbybin(ontdata,6)
  p8<-createhistforwcstbybin(ontdata,7)
  p9<-createhistforwcstbybin(ontdata,8)
  
  p<-arrangeGrob(p2,p3,p4,p5,p6,p7,p8,p9,ncol=4)
  return(list("plot_1"=p, "function_name"="ontologies_w_st_by_bin"))
}

createhistforwcstbyreasoner<-function(ontdata,reasoner){
  p<-ggplot(ontdata[ontdata$param_reasoner.x==reasoner&ontdata$wc_st_bin>0,], aes(x=wc_st_bin, fill=reasoner.x)) + geom_histogram(binwidth=1) 
  p<- p + theme(legend.position="none") + xlim(1,8)
  p<- p + scale_x_discrete(breaks=c("v.easy","easy"),name="") 
  return(p)
}

createhistforwcstbybin<-function(ontdata,bin,lim=50){
  flevels<-levels(as.factor(ontdata$param_reasoner.x))
  title<-binlabel(bin)
  p<-ggplot(ontdata[ontdata$wc_st_bin==bin,], aes(x=param_reasoner.x, fill=wc_st_bin)) + geom_histogram(binwidth=1,fill="black") 
  p<- p + theme(plot.margin=unit(c(0,0,0,0), "cm"),plot.title = element_text(lineheight=.8, face="bold"),legend.position="none",axis.title.y = element_blank(),axis.text.x  = element_text(size=15),axis.text.y  = element_text(size=15)) + scale_x_discrete(limits=flevels,name="")
  p<-p+ylim(0,lim) + stat_bin(binwidth=1, geom="text", aes(label=..count..), vjust=-0.3)
  p<-p+ ggtitle(title)
  return(p)
}

binlabel<-function(x) {
  if(x==0){
    return("No test")
  }
  else if(x==1){
    return("Trivial")
  }
  else if(x==2){
    return("Very Easy")
  }
  else if(x==3){
    return("Easy")
  }
  else if(x==4){
    return("Medium Easy")
  }
  else if(x==5){
    return("Medium")
  }
  else if(x==6){
    return("Medium Hard")
  }
  else if(x==7){
    return("Hard")
  }
  else if(x==8){
    return("Very Hard")
  }
  else {
    return("UNKOWN")
  }
}

binbytesttype<-function(c) {
  for(i in 1:length(c)) {
    if(is.na(c[i])){
      return(NA)
    }
    if(c[i]>0) {
      return(length(c)-(i-1));
    }
  }
  return(0)
}

cade_filter<-function(ontdata,out) {
  ontdata$testfilter<-ontdata$sat_ct_total_01_veryhard+ontdata$sat_ct_total_02_hard+ontdata$sat_ct_total_03_mediumhard+ontdata$sat_ct_total_04_medium
  filteredontdata<-ontdata[ontdata$testfilter>0&ontdata$owl2_dl==1,]
  #filteredontdata[c("reasoner.x","filename",sat_ct_total_numeric)]
  for(r in unique(filteredontdata$param_reasoner.x)) {
    subs<-subset(filteredontdata,param_reasoner.x==r)
    dirout<-paste(out,r,sep="")
    dir.create(dirout)
    fileout<-paste(dirout,"/filteredset.txt",sep="")
    write(unique(subs$filename),fileout,sep="\n")
  }
  write(unique(filteredontdata$filename),paste(out,"filteredfiles_all.txt",sep=""),sep="\n")
}

get_cade_filelist_filtered<-function(ontdata,dir,testlim=5000) {
  files<-c()
  for(r in unique(ontdata$reasoner)) {
    for(o in unique(ontdata$ontname)) {
      ontdata_sub<-subset(ontdata,ontdata$reasoner==r&ontdata$filename==paste("m_bottom_",o,sep=""))
      if(nrow(ontdata_sub)>0) {
        rname<-ontdata_sub$param_reasoner[1]
        sat_ct_total<-as.numeric(ontdata_sub$sat_ct_total[1])
        if(sat_ct_total<testlim){
          print(paste("Adding ",r,rname,o))
          file<-paste(dir,rname,"_",o,".RData",sep="")
          files<-c(files,file)
        }
      }
    }
  }
  return(files)
}

gethardesttest<-function(stdata,runid){
  r<-c()
  for(id in runid) {
    tests<-stdata[stdata$experiment_runid==id,]$testduration
    if(length(tests)>0) {
      r<-c(r,(max(tests)))
    }
    else {
      r<-c(r,NA)
    }
  }
  return(r)
}

createst_rat_plot<-function(ontdata_wt){
  ontdata_wt<-ontdata_wt[with(ontdata_wt, order(rat_sum_sat)), ]
  ontdata_wt$id<-as.factor(1:nrow(ontdata_wt))
  ontdata_wt_relont<- ontdata_wt[c("id","reasoner.x","rat_sum_sat","rat_max_sat")]
  row.names(ontdata_wt_relont)<-ontdata_wt_relont$id
  
  ontdata_wt_relont_melt <- melt(ontdata_wt_relont,id.vars=c("id","reasoner.x"))
  
  g<-ggplot(ontdata_wt_relont_melt, aes(x=rev(reorder(id,id)), y=value, group=variable)) + geom_line() +theme(legend.position=c(0.9,0.5),plot.margin=unit(c(0,0,0,0), "cm"),axis.title.y = element_blank(),axis.title.x = element_blank(),axis.text.y  = element_text(size=15),axis.text.x  = element_text(size=15))+ scale_x_discrete(breaks=c(20,40,60,80),labels=c(20,40,60,80)) +ylim(0,100) #geom_area(aes(colour = variable, fill= variable), position = 'stack')   
  return(g)
}

createstdistplot<-function(stdata,lim=NA){
  options(scipen="1000")
  relsat<-c("filename","reasoner","sub","super","pos")
  relstdata <- stdata[c(relsat,"hardcatname")]
  reshapedst <- melt(relstdata,id.vars = relsat)
  levels=as.factor(c("Trivial","Very Easy","Easy","Medium Easy","Medium","Medium Hard","Hard","Very Hard"))
  p<-ggplot(reshapedst, aes(x=value, fill=pos)) + geom_histogram(binwidth=1, position="dodge") + scale_y_log10() + scale_x_discrete(limits=levels,labels=c("T","VE","E","ME","MD","MH","H","VH")) 
  p<-p+ scale_fill_grey()+theme(legend.position=c(0.9,0.5),plot.margin=unit(c(0,0,0,0), "cm"),axis.title.y = element_blank(),axis.title.x = element_blank(),axis.text.x  = element_text(size=15),axis.text.y  = element_text(size=15))
  #cade_filter(ontdata,out)
  if(!is.na(lim)) {
    p<-p+ scale_y_log10(limits=c(1,lim))
  }
  return(p)
}

truefalse_to_negpos <- function(var, value){
  value <- as.character(value)
  if (var=="pos") { 
    value[value=="1"] <- "Positive"
    value[value=="0"]   <- "Negative"
  }
  return(value)
}

get_ontology_short_name<-function(o){
  return(sub("^(.*?)[.].*", "\\1", o))
}

export_stdata<-function(ontdata,stdata){
  for(r in unique(ontdata$reasoner)) {
    for(o in unique(ontdata$ontname)) {
      ontdata_sub<-subset(ontdata,ontdata$reasoner==r&ontdata$ontname==o)
      if(nrow(ontdata_sub)>0) {
      filenames<-unique(ontdata_sub$filename)
      rname<-ontdata_sub$param_reasoner[1]
      stdata_sub<-subset(stdata,stdata$reasoner==r&(stdata$filename %in% filenames))
      print(paste("Saving ",r,rname,o,nrow(stdata_sub)))
      
      save(stdata_sub, file = paste(rname,"_",o,".RData",sep=""))
      }
    }
  }
}