exportplots <- function(p,extension="pdf") {
  #tryCatch(ggsave(filename = gsub(pattern = "\\[|\\]|\\|",replacement = "-",x = paste(out.dir.plot,p$function_name,".jpg",sep="")),,plot = p$plot_1),error=function(e) {print(i);})
  ggsave(filename = gsub(pattern = "\\[|\\]|\\|",replacement = "-",x = paste(out.dir.plot,p$function_name,".",extension,sep="")),,plot = p$plot_1)
  #ggsave(filename = paste(out.dir.plot,p$function_name,".png",sep=""),plot = p$plot_1,width = 210, height = 297, units = "mm")
}

setplotdir <- function(x) {
  assign("out.dir.plot", paste("out/",x,"/",sep=""), envir = .GlobalEnv)
  dir.create(out.dir.plot)
}

lplot<-function(p,name){
  return(list("plot_1"=p, "function_name"=name))
}

emptyplot <- function() {
  p.plot <- ggplot(data.frame()) + geom_point() + xlim(0, 10) + ylim(0, 100)
  return(list("plot_1"=p.plot, "function_name"="emptyplot"))
}

exportplotly <- function(plot) {
  library(plotly)
  py <- plotly(username="matentzn", key="dpa87w0w1k")
  out <- py$ggplotly(plot)
  plotly_url <- out$response$url
  return(plotly_url)
}

boxplotmultiplecolumns_logy <- function(data,columns,name) {
  x <- data[columns]
  y <- x[complete.cases(x),]
  m1 <- melt(y[columns])
  p.plot <- ggplot(m1,aes(x = variable,y = value)) + geom_boxplot() + coord_flip() + 
    scale_y_continuous(trans = log10_trans(),
                       breaks = trans_breaks("log10", function(x) 10^x),
                       labels = trans_format("log10", math_format(10^.x)))
  return(list("plot_1"=p.plot, "function_name"=name)) 
}

boxplotmultiplecolumns <- function(data,columns,name) {
  x <- data[columns]
  y <- x[complete.cases(x),]
  m1 <- melt(y[columns])
  p.plot <- ggplot(m1,aes(x = variable,y = value)) + geom_boxplot() + coord_flip()
  return(list("plot_1"=p.plot, "function_name"=name)) 
}

plotcorrelation <- function(data,relevant,name) {
  x <- data[relevant]
  data_filtered <- x[complete.cases(x),]
  p.plot <- qplot(x=Var1, y=Var2, data=melt(cor(data_filtered)), fill=value, geom="tile")
  return(list("plot_1"=p.plot, "function_name"=name)) 
}

plotlinegraphs <- function(data,relevant,name,x,y) {
  data[, relevant] <- sapply(data[, relevant], as.numeric)
  x <- data[relevant]
  data_filtered <- x[complete.cases(x),]
  data_filtered[, relevant] <- sapply(data[, relevant], as.numeric)
  p.plot <- ggplot(data=df, aes(x=x, y=y, group=1)) + geom_line()
  return(list("plot_1"=p.plot, "function_name"=name)) 
}
plotsinglevariablelinegraphs <- function(data,column) {
  name <- paste(column,"_single",sep = "")
  x <- sort(data[,column])
  p.plot <- qplot(seq_along(x), x)
  return(list("plot_1"=p.plot, "function_name"=name)) 
}

plotbarchart <- function(df,x1,y1,name) {
  p.plot <- ggplot(df, aes_string(x = x1,y=y1)) + geom_bar(stat = "identity")
  return(list("plot_1"=p.plot, "function_name"=name))
}
plotbarchart_horiz <- function(df,x1,y1,name) {
  p.plot <- ggplot(df, aes_string(x = x1,y=y1)) + geom_bar(stat = "identity") + coord_flip()
  return(list("plot_1"=p.plot, "function_name"=name))
}
plotlogaxhistogram <- function(data,name) {
  df <- data[name]
  p.plot <- ggplot(df, aes_string(name)) + scale_x_log10(breaks=c(10,100,1000,10000,100000,1000000,10000000),labels=c(10,100,1000,10000,100000,1000000,10000000)) + geom_histogram(binwidth=log10(2))
  return(list("plot_1"=p.plot, "function_name"=paste("hist",name,sep="")))
}

plothistogram <- function(data,name,plotname) {
  df <- data[name]
  p.plot <- ggplot(df, aes_string(name)) + geom_histogram()
  return(list("plot_1"=p.plot, "function_name"=paste("hist",name,"_",plotname,sep="")))
}

plotlogaxhistogramoverlap <- function(data,name1,name2) {
  df <- data[c(name1,name2)]
  p.plot <- ggplot(df, aes_string(name)) + scale_x_log10(breaks=c(10,100,1000,10000,100000,1000000,10000000)) + geom_histogram(binwidth=log10(2))
  return(list("plot_1"=p.plot, "function_name"=paste("hist",name,sep="")))
}

generateHistogramLogaxOverlap <- function(datavector1,datavector2,outdir,plotname,xlabel){
  hist1<-hist(log10(datavector1),breaks=log10(breakpointsx),freq=TRUE, main=plotname,xaxt="n",xlab=xlabel,col=rgb(139,139,139,maxColorValue=255))
  hist2<-hist(log10(datavector2),add=T,breaks=log10(breakpointsx),freq=TRUE, main=plotname,xaxt="n",xlab=xlabel,col=rgb(169,169,169,maxColorValue=255))
  axis(1, at=0:7, labels=breakpointsx)  
  return(list("hist1"=hist1,"hist2"=hist2))
}

# Multiple plot function
#
# ggplot objects can be passed in ..., or to plotlist (as a list of ggplot objects)
# - cols:   Number of columns in layout
# - layout: A matrix specifying the layout. If present, 'cols' is ignored.
#
# If the layout is something like matrix(c(1,2,3,3), nrow=2, byrow=TRUE),
# then plot 1 will go in the upper left, 2 will go in the upper right, and
# 3 will go all the way across the bottom.
# from http://www.cookbook-r.com/Graphs/Multiple_graphs_on_one_page_(ggplot2)/
multiplot <- function(..., plotlist=NULL, file, cols=1, layout=NULL) {
  require(grid)
  
  # Make a list from the ... arguments and plotlist
  plots <- c(list(...), plotlist)
  
  numPlots = length(plots)
  
  # If layout is NULL, then use 'cols' to determine layout
  if (is.null(layout)) {
    # Make the panel
    # ncol: Number of columns of plots
    # nrow: Number of rows needed, calculated from # of cols
    layout <- matrix(seq(1, cols * ceiling(numPlots/cols)),
                     ncol = cols, nrow = ceiling(numPlots/cols))
  }
  
  if (numPlots==1) {
    print(plots[[1]])
    
  } else {
    # Set up the page
    grid.newpage()
    pushViewport(viewport(layout = grid.layout(nrow(layout), ncol(layout))))
    
    # Make each plot, in the correct location
    for (i in 1:numPlots) {
      # Get the i,j matrix positions of the regions that contain this subplot
      matchidx <- as.data.frame(which(layout == i, arr.ind = TRUE))
      
      print(plots[[i]], vp = viewport(layout.pos.row = matchidx$row,
                                      layout.pos.col = matchidx$col))
    }
  }
}

ggplot_theme_default<-function(p){
  p<-p+theme(legend.position="none",plot.margin=unit(c(0.1,0.1,0.1,0.1), "cm"),axis.title.y = element_blank(),axis.title.x = element_blank(),axis.text.x  = element_text(size=12),axis.text.y  = element_text(size=13))
  return(p)
}