corpusmetrics_numeric <- c("filesize","abox_size_incl" ,"ax_complexrhs_ct_incl" ,"tboxrbox_size_incl" ,"logical_axiom_count" ,"tboxrbox_size" ,"avg_size_complexrhs" ,"tbox_size_incl_before","dt_notbuiltin_ct" ,"avg_size_complexrhs_incl" ,"dt_builtin_ct_incl" ,"obj_property_count_incl" ,"ax_complexrhs_ct","class_count" ,"indiv_count_incl","indiv_count" ,"dataproperty_count_incl" ,"class_count_incl","annotation_property_count" ,"dataproperty_count" ,"owl2" ,"rdfs" ,"obj_property_count" ,"owl2_ql" ,"logical_axiom_count_incl"  ,"owl2_rl","anno_ct","rule_ct_incl","datatypes_count" ,"dt_builtin_ct","signature_size" ,"datatypes_count_incl" ,"rbox_size_incl","tbox_size_incl" ,"tbox_size" ,"rbox_size","rule_ct"  ,"signature_size_incl" ,"owl2_dl" ,"abox_size","owl2_el")

mergeonfilename <- function(d1,d2) {
  data <- merge(x=d1, y=d2, by.x="filename", by.y="filename")
  return(data)
}

loadontologydata <- function(filename) {
  data <- read.csv(paste("data/",filename,sep=""),colClasses = "character")
  data[, corpusmetrics_numeric] <- sapply(data[, corpusmetrics_numeric], as.numeric)
  return(data)
}

colPattern<-function(df,pattern) {
  return(names(df[grepl(pattern,names(df))]))
}

pc<-function(i,n) {
  round((i/n)*100,2)
}

mergecolumnstodataframe <- function(c1,c2,names) {
  datav <- data.frame(c1)
  datav$c2 <- c2
  names(datav) <- names
  return(datav)
}

dropColumnsFromDF<-function(df,cols){
  return(df[ , -which(names(df) %in% cols)])
}

getdatainseconds <- function(data,columns) {
  data[columns] <- as.data.frame(lapply(data[columns],FUN = function(x) {sapply(x,FUN=getseconds)}))
  return(data)
}
getseconds <- function(x) {
  if(is.numeric(x)) {
    x/1000000000
  }
  else{
    x
  }
}

sampleMedian <- function(df,metric,id) {
  #df<-df_modtech
  df$sm_id<-1:nrow(df)
  data<-df  
  data$xxx_id<-data[[id]]
  agg<-aggregate(formula(paste0(metric,"~",id, sep="")), data = data, FUN = median)
  agg$xxx_median<-agg[[metric]]
  data<-merge(data,agg[c(id,"xxx_median")],by=c(id),all.x=TRUE)
  data$xxx_diff<-abs(data$xxx_median-data[[metric]])
  agg2<-aggregate(formula(paste0("xxx_diff","~",id, sep="")), data = data, FUN = min)
  agg2$sample<-TRUE
  data<-merge(data,agg2[c(id,"xxx_diff","sample")],by=c(id,"xxx_diff"),all.x=TRUE)
  data$sample[is.na(data$sample)] <- FALSE
  data<-data[data$sample,]
  set.seed(1)
  rndid <- with(data, ave(sm_id, xxx_id, FUN=function(x) {sample.int(length(x))}))
  data<-data[rndid<=1,]
  df<-merge(df,data[c("sample","sm_id")],by="sm_id",all.x=TRUE)
  df$sample[is.na(df$sample)] <- FALSE
  return(df[!(names(df) %in% "sm_id")])
}

getdataintimeunit <- function(data,columns,funct) {
  data[columns] <- as.data.frame(lapply(data[columns],FUN = function(x) {sapply(x,FUN=funct)}))
  return(data)
}

getmilliseconds <- function(x) {
  if(is.numeric(x)) {
    x/1000000
  }
  else{
    x
  }
}

foldChangeNormalised <- function(x,y) {
  res <- (y/x)
  if(is.na(res)) {
    NA
  }
  else {
  if((res<1) & (res>(-1))) {
    (1/res)*(-1)
  }
  else {
    res
  }
  }
}

foldChange <- function(x,y) {
  (y/x)
}

normalizedZeroOne<-function(x) {
  minx<-min(x,na.rm = TRUE)
  maxx<-max(x,na.rm = TRUE)
  y<-c()
  for(i in x) {
    if(!is.na(i)) {
      norm<-((i-minx)/(maxx-minx))
      y<-c(y,norm)
    }
    else {
      y<-c(y,NA)
    }
  }
  return(y)
}

percentDifference <- function(x,y) {
  ((x - y)/((x+y)/2))
}

percentChange <- function(x,y) {
  (x-y)/y
}

list.dirs <- function(path=".", pattern=NULL, all.dirs=FALSE,
                      full.names=FALSE, ignore.case=FALSE) {
  
  all <- list.files(path, pattern, all.dirs,
                    full.names, recursive=FALSE, ignore.case)
  all[file.info(all)$isdir]
}

createNewSample <- function(filenames,sourcedir,targetdir) {
  paths <- paste(sourcedir,filenames,sep="")
  to<- paste(targetdir,filenames,sep="")
  file.copy(from = paths,to=to)
}

val_pc<-function(v1,v2){
  return(round((v1/v2)*100,2))
}

std_error <- function(y){ 
  x<-y[!is.na(y)]
  se<-sd(x)/sqrt(length(x))
  return(se)
}

co.var <- function(x) ( sd(x, na.rm = TRUE)/mean(x, na.rm = TRUE) )

prepareworkspace<-function(name){
  library(ggplot2)
  library(reshape2)
  library(grid)
  library(gridExtra)
  library(scales)
  library(plyr)
  library(xtable)
  library(data.table)
  source("plot.R") 
  source("phase.R")
  source("corpus.R")
  setplotdir(name) 
}

df2csv<-function(df,filename){
  write.csv(df,paste(getwd(),"/",out.dir.plot,filename,sep=""))
}

preprocesscorpusmetadata<-function(raw_data_f) {
  prepro_data_f<-paste(getwd(),"/",out.dir.plot,"tmp_df_prepro.csv",sep="")
  unlink(prepro_data_f)
  command<-paste("phd_metrics_prepro.jar",raw_data_f,prepro_data_f)
  system(paste("java","-jar",command))
  metrics<-read.csv(file=prepro_data_f,head=TRUE,sep=",")
  return(metrics)
}

mkcolnumeric<-function(df,col){
  df[, col] <- sapply(df[, col], as.numeric)
  return(df)
}

create_corpus_dataset<-function(ontmetadatafile){
  #Corpus Metadata#  
  corpus_metrics<-read.csv(ontmetadatafile,colClasses = "character")
  corpus_metrics$o<-get_ontology_short_name(corpus_metrics$filename)
  corpus_metrics_prepro<-preprocesscorpusmetadata(ontmetadatafile)
  corpus_metrics_prepro$o<-get_ontology_short_name(corpus_metrics_prepro$filename)
  corpus_metrics<-mkcolnumeric(corpus_metrics,c("logical_axiom_count_incl"))
  
  ontnonempty<-corpus_metrics[corpus_metrics$tboxrbox_size_incl>0,]$o
  
  ontmetadata <- corpus_metrics[corpus_metrics$o %in% ontnonempty,]
  ontmetadata_pre <- corpus_metrics_prepro[corpus_metrics_prepro$o %in% ontnonempty,]
  corpus_md<-merge(ontmetadata,ontmetadata_pre,by="o")
  corpus_md<-generateProfileMetrics(corpus_md)
  corpus_md<-sizeBin(corpus_md)
  corpus_md$r<-corpus_md$param_reasoner
  return(corpus_md)
}

sizeBin<-function(df){
  df$sizebin<-ifelse(df$logical_axiom_count_incl<1,"empty",ifelse(df$logical_axiom_count_incl<10,"very small",ifelse(df$logical_axiom_count_incl<100,"small",ifelse(df$logical_axiom_count_incl<1000,"medium",ifelse(df$logical_axiom_count_incl<10000,"large",ifelse(df$logical_axiom_count_incl<100000,"very large","huge"))))))
  levels(df$sizebin) <- rev(c("empty","very small","small","medium","large","very large","huge"))
  return(df)
}

getOrdering<-function(x) {
  s<-sort(x, decreasing = FALSE, na.last = TRUE)
  out<-c()
  for(i in x) {
    o<-5
    if(!is.na(i)){
      o<-which(s==i)
      if(o>3){
        o<-4
      }
    }
    out<-c(out,o)
  }
  return(out)
}

asinh_trans <- function(){
  trans_new(name = 'asinh', transform = function(x) asinh(x), 
                             inverse = function(x) sinh(x))
}

illegallogtransform<-function(x){
  y<-ifelse(x>=0,TRUE,FALSE)
  z<-abs(x)
  z<-log10(z)
  z<-ifelse(y,z,-z)
  return(z)
}